@@ -18,7 +18,6 @@ pass_config = click.make_pass_decorator(Config, ensure=True)
 @click.group(cls=AliasedGroup)
 @click.option("-f", "--force", is_flag=True)
 @click.option("-v", "--verbose", is_flag=True)
-@click.option("--version", is_flag=True)
 @click.option(
     "-xs",
     "--restrict-setting",
@@ -42,7 +41,6 @@ def cli(
     restrict_setting,
     restrict_docker_compose,
     chdir,
-    version,
 ):
     config.force = force
     config.verbose = verbose
@@ -85,14 +83,18 @@ def completion(execute):
         ):
             content += [f"\n{line}\n"]
             click.secho(
-                f"Inserted successfully\n{line}" "\n\nPlease restart you shell."
+                f"Inserted successfully\n{line}"
+                "\n\nPlease restart you shell."
             )
             rc_file.write_text("\n".join(content))
         else:
             click.secho("Nothing done - already existed.")
     else:
         click.secho(
-            "\n\n" f"Insert into {rc_file}\n\n" f"echo '{line}' >> {rc_file}" "\n\n"
+            "\n\n"
+            f"Insert into {rc_file}\n\n"
+            f"echo '{line}' >> {rc_file}"
+            "\n\n"
         )
     sys.exit(0)
 
@@ -105,7 +107,9 @@ def version(config):
     version = _get_version()
 
     images_sha = subprocess.check_output(
-        ["git", "log", "-n1", "--format=%H"], encoding="utf8", cwd=config.dirs["images"]
+        ["git", "log", "-n1", "--format=%H"],
+        encoding="utf8",
+        cwd=config.dirs["images"],
     ).strip()
     images_branch = subprocess.check_output(
         ["git", "rev-parse", "--abbrev-ref", "HEAD"],
