@@ -1,4 +1,7 @@
 import traceback
+import socket
+
+import netifaces
 import threading
 from .tools import bashfind
 import json
@@ -9,8 +12,6 @@ import time
 import collections
 import grp
 import base64
-import pwd
-from contextlib import contextmanager
 import platform
 from pathlib import Path
 import importlib.util
@@ -26,6 +27,7 @@ import copy
 import click
 from . import module_tools
 from . import tools
+from .tools import is_interactive
 from .tools import update_setting
 from .tools import __replace_all_envs_in_str
 from .tools import __running_as_root_or_sudo
@@ -34,7 +36,6 @@ from .tools import __try_to_set_owner
 from .tools import whoami
 from .tools import abort
 from .tools import _get_version
-from .tools import rsync
 from .tools import __read_file
 from .tools import get_docker_version
 from .tools import _parse_yaml
@@ -44,6 +45,8 @@ from .odoo_config import MANIFEST
 from .tools import execute_script
 from .tools import ensure_project_name
 from .tools import update_docker_service
+from .tools import autocleanpaper
+from .tools import sync_folder
 
 import inspect
 import os
@@ -61,7 +64,9 @@ def composer(config):
 
 
 @composer.command()
-@click.option("--full", is_flag=True, help="Otherwise environment is shortened.")
+@click.option(
+    "--full", is_flag=True, help="Otherwise environment is shortened."
+)
 @click.argument("service-name", required=False)
 @pass_config
 @click.pass_context
@@ -116,14 +121,23 @@ def _get_arch():
 @click.option("--headless", is_flag=True, help="Dont start a web-server")
 @click.option("--devmode", is_flag=True)
 @click.option(
-    "-c", "--additional_config", help="Base64 encoded configuration like in settings"
+    "-c",
+    "--additional_config",
+    help="Base64 encoded configuration like in settings",
+)
+@click.option(
+    "-cR",
+    "--additional_config_raw",
+    help="like ODOO_DEMO=1;RUN_PROXY=0; you can pass internally a 'dict' here, which is converted",
+)
+@click.option(
+    "--images-url", help="default: https://github.com/marcwimmer/odoo"
 )
-@click.option("-cR", "--additional_config_raw", help="like ODOO_DEMO=1;RUN_PROXY=0")
-@click.option("--images-url", help="default: https://github.com/marcwimmer/odoo")
 @click.option("-I", "--no-update-images", is_flag=True)
 @click.option("--no-gimera-apply", is_flag=True)
 @click.option(
-    "--docker-compose", help="additional docker-compose files, separated by colon :"
+    "--docker-compose",
+    help="additional docker-compose files, separated by colon :",
 )
 @click.option("-s", "--include-src", help="include source")
 @pass_config
@@ -149,19 +163,22 @@ def do_reload(
     from .module_tools import NotInAddonsPath
 
     if headless and proxy_port:
-        click.secho(("Proxy Port and headless together not compatible."), fg="red")
+        click.secho(
+            ("Proxy Port and headless together not compatible."), fg="red"
+        )
         sys.exit(-1)
 
     if not no_update_images:
         _download_images(config, images_url)
     config.TARGETARCH = _get_arch()
 
-    click.secho(f"Current Project Name: {config.project_name}", bold=True, fg="green")
+    click.secho(
+        f"Current Project Name: {config.project_name}", bold=True, fg="green"
+    )
     SETTINGS_FILE = config.files.get("settings")
     if SETTINGS_FILE and SETTINGS_FILE.exists():
         SETTINGS_FILE.unlink()
 
-
     additional_config_file = None
     try:
         if additional_config:
@@ -169,7 +186,9 @@ def do_reload(
             additional_config_text = base64.b64decode(additional_config)
             additional_config_file.write_bytes(additional_config_text)
             additional_config = MyConfigParser(additional_config_file)
-            click.secho(f"Additional config provided in {additional_config_file}:")
+            click.secho(
+                f"Additional config provided in {additional_config_file}:"
+            )
             for line in additional_config_text.decode("utf-8").split("\n"):
                 click.secho("\t" + line)
 
@@ -182,7 +201,10 @@ def do_reload(
             "\n".join(
                 filter(
                     lambda x: not x.startswith("#"),
-                    map(lambda x: x.strip(), (additional_config or "").splitlines()),
+                    map(
+                        lambda x: x.strip(),
+                        (additional_config or "").splitlines(),
+                    ),
                 )
             )
         ) + "\n"
@@ -194,20 +216,22 @@ def do_reload(
         if docker_compose:
             docker_compose = docker_compose.split(":")
 
+        if include_src:
+            additional_config += "\nSRC_EXTRA=0"
+
         try:
             internal_reload(
-                ctx,
-                config,
-                db,
-                demo,
-                devmode,
-                headless,
-                proxy_port,
-                mailclient_gui_port,
-                additional_config,
+                ctx=ctx,
+                config=config,
+                db=db,
+                demo=demo,
+                devmode=devmode,
+                headless=headless,
+                proxy_port=proxy_port,
+                mailclient_gui_port=mailclient_gui_port,
+                additional_config=additional_config,
                 additional_docker_configuration_files=docker_compose,
                 no_gimera_apply=no_gimera_apply,
-                include_src=include_src,
             )
         except NotInAddonsPath as ex:
             abort(str(ex))
@@ -217,10 +241,47 @@ def do_reload(
             additional_config_file.unlink()
 
 
+def get_local_ips():
+    ips = []
+    for iface in netifaces.interfaces():
+        addrs = netifaces.ifaddresses(iface).get(netifaces.AF_INET, [])
+        for addr in addrs:
+            ip = addr.get("addr")
+            if ip and not ip.startswith("127."):
+                ips.append(ip)
+    return ips
+
+
+def choose_ip(ips):
+    import inquirer
+
+    choices = ips + ["Enter manually..."]
+    questions = [
+        inquirer.List(
+            "ip",
+            message="Choose an IP address",
+            choices=choices,
+        )
+    ]
+    answers = inquirer.prompt(questions)
+    if answers["ip"] == "Enter manually...":
+        manual = inquirer.text(message="Enter IP address:")
+        return manual
+    return answers["ip"]
+
+
 def get_arch():
     return platform.uname().machine  # aarch64
 
 
+def setup_apt_cacher_host(ctx, settings):
+    ips = list(sorted(get_local_ips()))
+    ip = choose_ip(ips)
+    if not is_interactive():
+        abort("Please define system wide APT_PROXY_IP")
+    ctx.invoke(setting, name="APT_PROXY_IP", value=ip, no_reload=False)
+
+
 def internal_reload(
     ctx,
     config,
@@ -228,25 +289,26 @@ def internal_reload(
     demo,
     devmode,
     headless,
-    local,
     proxy_port,
     mailclient_gui_port,
     additional_config=None,
     additional_docker_configuration_files=None,
     no_gimera_apply=False,
-    include_src=False,
 ):
     threading.currentThread().config = config
     ensure_project_name(config)
-    additional_docker_configuration_files = additional_docker_configuration_files or []
+    additional_docker_configuration_files = (
+        additional_docker_configuration_files or []
+    )
     defaults = {
         "config": config,
         "db": db,
         "demo": demo,
-        "LOCAL_SETTINGS": "1" if local else "0",
-        "CUSTOMS_DIR": config.WORKING_DIR,
+        "HOST_CUSTOMS_DIR": config.WORKING_DIR,
         "QUEUEJOB_CHANNELS_FILE": config.files["queuejob_channels_file"],
     }
+    from .myconfigparser import MyConfigParser
+
     if devmode:
         defaults["DEVMODE"] = 1
     if demo:
@@ -267,10 +329,15 @@ def internal_reload(
         defaults["ROUNDCUBE_PORT"] = mailclient_gui_port
 
     if additional_config:
-        for key in additional_config.keys():
-            defaults[key] = additional_config[key]
+        with autocleanpaper() as paper:
+            paper.write_text(additional_config)
+            addconfig = MyConfigParser(paper)
+            for key in addconfig.keys():
+                defaults[key] = addconfig[key]
+            del addconfig
 
-        click.secho("Additional config: {defaults}")
+        if defaults:
+            click.secho(f"Additional config: {defaults}")
 
     ODOO_VERSION = str(MANIFEST()["version"]).split(".")[0]
     KEY_ODOO_VERSION = f"RUN_ODOO_VERSION_{ODOO_VERSION}"
@@ -284,18 +351,22 @@ def internal_reload(
         **defaults,
         additional_docker_configuration_files=additional_docker_configuration_files,
         no_gimera_apply=no_gimera_apply,
-        include_src=include_src,
+        ctx=ctx,
     )
 
     _execute_after_reload(config)
 
     # update a possible robot file also
-    Commands.invoke(ctx, 'robot:make-var-file')
+    Commands.invoke(ctx, "robot:make-var-file")
 
     if float(ODOO_VERSION) >= 17.0:
-        if any(config.ODOO_PYTHON_VERSION.startswith(x) for x in ['3.9.', '3.8.', '3.7.']):
+        if any(
+            config.ODOO_PYTHON_VERSION.startswith(x)
+            for x in ["3.9.", "3.8.", "3.7."]
+        ):
             abort("Invalid python version - needs at least 3.10")
 
+
 def _execute_after_reload(config):
     execute_script(
         config,
@@ -310,6 +381,7 @@ def _set_defaults(config, defaults):
     defaults["project_name"] = config.project_name
     defaults["WODOO_VERSION"] = _get_version()
 
+
 def _do_compose(
     config,
     db="",
@@ -317,12 +389,12 @@ def _do_compose(
     additional_docker_configuration_files=None,
     no_gimera_apply=False,
     include_src=False,
+    ctx=None,
     **forced_values,
 ):
     """
     builds docker compose, proxy settings, setups odoo instances
     """
-    from .myconfigparser import MyConfigParser
     from .settings import _export_settings
 
     rows = []
@@ -341,7 +413,7 @@ def _do_compose(
 
     defaults = {}
     _set_defaults(config, defaults)
-    setup_settings_file(config, db, demo, **defaults)
+    setup_settings_file(ctx, config, db, demo, **defaults)
     _export_settings(config, forced_values)
     _prepare_filesystem(config)
     _execute_after_settings(config)
@@ -349,14 +421,88 @@ def _do_compose(
     _prepare_yml_files_from_template_files(
         config, additional_docker_configuration_files
     )
-    _merge_odoo_dockerfile(config, include_src=include_src)
+    _merge_odoo_dockerfile(config)
+    _copy_all_dockerfiles_to_run_dir_and_set_dockerfile_in_dockercompose(
+        config
+    )
+
+    _redo_if_settings_missing(ctx, config)
 
     click.echo("Built the docker-compose file.")
 
 
+def _copy_all_dockerfiles_to_run_dir_and_set_dockerfile_in_dockercompose(
+    config,
+):
+    import yaml
+
+    from .myconfigparser import MyConfigParser
+
+    settings = MyConfigParser(config.files["settings"])
+
+    content = config.files["docker_compose"].read_text()
+    content = yaml.safe_load(content)
+    dockerfile1 = None
+    images_dir = config.dirs["images"]
+    to_del = set()
+    for service_name in content["services"]:
+        service = content["services"][service_name]
+        if service.get("image"):
+            continue
+        buildcontext = service.get("build", {}).get("context")
+        if not buildcontext:
+            buildcontext = images_dir / service_name
+        if "build" not in service:
+            service["build"] = {
+                "context": buildcontext,
+                "args": {},
+            }
+
+        delete_source = False
+        if service.get("build", {}).get("dockerfile"):
+            src_dockerfile = Path(service["build"]["dockerfile"])
+            if not str(src_dockerfile).startswith("/"):
+                src_dockerfile = Path(buildcontext) / src_dockerfile
+        else:
+            src_dockerfile = Path(service["build"]["context"]) / "Dockerfile"
+        delete_source = str(src_dockerfile).startswith(str(config.dirs["run"]))
+        trgt_dockerfile = config.dirs["run"] / "Dockerfiles" / service_name
+        trgt_dockerfile.parent.mkdir(parents=True, exist_ok=True)
+        src = src_dockerfile.read_text()
+        src = _replace_docker_snippets(config, src)
+        trgt_dockerfile.write_text(src)
+        if delete_source:
+            to_del.add(src_dockerfile)
+        service["build"]["dockerfile"] = str(trgt_dockerfile)
+    [x.unlink() for x in to_del]
+    content = yaml.dump(content, default_flow_style=False)
+    config.files["docker_compose"].write_text(content)
+
+
+def _replace_docker_snippets(config, dockerfilecontent):
+    for snippet in (config.dirs["images"] / "common_snippets").glob("*"):
+        content = snippet.read_text()
+        name = f"#___SNIPPET_{snippet.stem.upper()}___"
+        dockerfilecontent = dockerfilecontent.replace(name, content)
+    return dockerfilecontent
+
+
+def _redo_if_settings_missing(ctx, config):
+    from .myconfigparser import MyConfigParser
+
+    settings = MyConfigParser(config.files["settings"])
+    if not settings.get("APT_PROXY_IP"):
+        if settings.get("RUN_APT_CACHER") in ["1", ""]:
+            setup_apt_cacher_host(ctx, settings)
+
+
 def _download_images(config, images_url):
     from . import consts
 
+    click.secho("--------------------------------------------------")
+    click.secho("Downloading Odoo Images", fg="green", bold=True)
+    click.secho("--------------------------------------------------")
+
     if not config.dirs["images"].exists():
         subprocess.check_call(
             [
@@ -374,9 +520,7 @@ def _download_images(config, images_url):
     effective_branch = config.ODOO_IMAGES_BRANCH or consts.IMAGES_REPO_BRANCH
 
     if effective_branch and effective_branch != current_branch:
-        subprocess.check_call(
-            ["git", "fetch"], cwd=config.dirs["images"]
-        )
+        subprocess.check_call(["git", "fetch"], cwd=config.dirs["images"])
         subprocess.check_call(
             ["git", "checkout", effective_branch], cwd=config.dirs["images"]
         )
@@ -388,7 +532,16 @@ def _download_images(config, images_url):
         for _ in range(10):
             trycount += 1
             try:
-                subprocess.check_call(["git", "pull"], cwd=config.dirs["images"])
+                subprocess.check_call(
+                    [
+                        "git",
+                        "pull",
+                        "--rebase=false",
+                        "--autostash",
+                        "--quiet",
+                    ],
+                    cwd=config.dirs["images"],
+                )
             except Exception as ex:
                 if trycount < 5:
                     time.sleep(random.randint(5, 30))
@@ -445,7 +598,7 @@ def get_db_name(db, project_name):
     return db
 
 
-def setup_settings_file(config, db, demo, **forced_values):
+def setup_settings_file(ctx, config, db, demo, **forced_values):
     """
     Cleans run/settings and sets minimal settings;
     Puts default values in settings.d to override any values
@@ -457,8 +610,18 @@ def setup_settings_file(config, db, demo, **forced_values):
     vals["DBNAME"] = get_db_name(db, config.project_name)
     if demo:
         vals["ODOO_DEMO"] = "1" if demo else "0"
-    vals.update(forced_values)
 
+    # take PYTHON VERSION from MANIFEST
+    m = MANIFEST()
+    if not settings.get("ODOO_PYTHON_VERSION"):
+        if m.get("python_version", None):
+            settings["ODOO_PYTHON_VERSION"] = m["python_version"]
+
+    for k, v in m.get("settings", {}).items():
+        settings[k] = v
+
+    # at end:
+    vals.update(forced_values)
     for k, v in vals.items():
         if settings.get(k, "") != v:
             settings[k] = v
@@ -545,9 +708,11 @@ def _prepare_yml_files_from_template_files(
     # - also replace all environment variables
     _files = []
 
+    manifest = MANIFEST()
+    customs_dir = odoo_config.customs_dir()
     for dir in [
         config.dirs["images"],
-        odoo_config.customs_dir(),
+        customs_dir,
         Path("/etc/odoo/"),
     ]:
         if not dir.exists():
@@ -556,7 +721,14 @@ def _prepare_yml_files_from_template_files(
             _files += [dir]
         else:
             for file in bashfind(dir, "docker-compose*.yml"):
-                _files.append(file)
+                ignore = False
+                if dir == customs_dir:
+                    ignore = str(
+                        file.relative_to(customs_dir)
+                    ) in manifest.get("ignore_compose_files", [])
+
+                if not ignore:
+                    _files.append(file)
 
     if config.restrict and config.restrict.get("docker-compose"):
         _files += config.restrict["docker-compose"]
@@ -569,7 +741,8 @@ def _prepare_yml_files_from_template_files(
             if not d.exists():
                 if config.verbose:
                     click.secho(
-                        f"Hint: you may use configuration file {d}", fg="magenta"
+                        f"Hint: you may use configuration file {d}",
+                        fg="magenta",
                     )
                 continue
             if d.is_file():
@@ -595,7 +768,9 @@ def _prepare_yml_files_from_template_files(
     _files = _files2
     del _files2
 
-    _prepare_docker_compose_files(config, config.files["docker_compose"], _files)
+    _prepare_docker_compose_files(
+        config, config.files["docker_compose"], _files
+    )
 
 
 def __resolve_custom_merge(whole_content, value):
@@ -627,7 +802,10 @@ def __get_sorted_contents(paths):
             order = "99999999"
         else:
             order = (
-                content.split("manage-order")[1].split("\n")[0].replace(":", "").strip()
+                content.split("manage-order")[1]
+                .split("\n")[0]
+                .replace(":", "")
+                .strip()
             )
         order = int(order)
 
@@ -646,7 +824,9 @@ def __set_environment_in_services(content):
 
         file = "$HOST_RUN_DIR/settings"
         if not [x for x in service["env_file"] if x == file]:
-            if service.get("labels", {}).get("odoo_framework.apply_env", "1") not in [
+            if service.get("labels", {}).get(
+                "odoo_framework.apply_env", "1"
+            ) not in [
                 0,
                 "0",
                 "false",
@@ -680,7 +860,9 @@ def post_process_complete_yaml_config(config, yml):
 
     # set container name to service name (to avoid dns names with _1)
     for service in yml["services"]:
-        yml["services"][service]["container_name"] = f"{config.project_name}_{service}"
+        yml["services"][service][
+            "container_name"
+        ] = f"{config.project_name}_{service}"
         # yml['services'][service]['hostname'] = service # otherwise odoo pgcli does not work
 
     # set label from configuration settings starting with DOCKER_LABEL=123
@@ -747,7 +929,9 @@ def __run_docker_compose_config(config, contents, env):
                 cmdline2 = buildcmd(files[: i + 1])
                 try:
                     click.secho(f"Testing {file}...", fg="green")
-                    conf = subprocess.check_output(cmdline2, cwd=temp_path, env=d)
+                    conf = subprocess.check_output(
+                        cmdline2, cwd=temp_path, env=d
+                    )
                 except:
                     click.secho(f"{file}:\n", fg="yellow")
                     click.secho(file.read_text())
@@ -816,7 +1000,7 @@ def dict_merge(dct, merge_dct):
 
 def _prepare_docker_compose_files(config, dest_file, paths):
     from .myconfigparser import MyConfigParser
-    from .tools import abort, atomic_write
+    from .tools import atomic_write
     import yaml
 
     if not dest_file:
@@ -867,7 +1051,6 @@ def _explode_referenced_machines(contents):
     a service is referenced; this service is copied in its own file to match later that reference by its service
     name in docker compose config
     """
-    import yaml
 
     needs_explosion = {}
 
@@ -902,7 +1085,9 @@ def _apply_variables(config, contents, env):
     import yaml
 
     # add static yaml content to each machine
-    default_network = yaml.safe_load(config.files["config/default_network"].read_text())
+    default_network = yaml.safe_load(
+        config.files["config/default_network"].read_text()
+    )
 
     # extract further networks
     for content in contents:
@@ -959,7 +1144,9 @@ def toggle_settings(ctx, config):
     questions = [
         inquirer.Checkbox(
             "run",
-            message="What services to run? {}/{}".format(config.customs, config.dbname),
+            message="What services to run? {}/{}".format(
+                config.customs, config.dbname
+            ),
             choices=choices,
             default=default,
         )
@@ -976,7 +1163,7 @@ def toggle_settings(ctx, config):
 
 
 def _use_file(config, path):
-    from . import odoo_config
+    pass
 
     def check():
         if "etc" in path.parts:
@@ -1055,7 +1242,30 @@ def _use_file(config, path):
     return res
 
 
-def _merge_odoo_dockerfile(config, include_src):
+def get_docker_host_ip():
+    try:
+        result = subprocess.run(
+            [
+                "docker",
+                "network",
+                "inspect",
+                "bridge",
+                "--format",
+                "{{(index .IPAM.Config 0).Gateway}}",
+            ],
+            capture_output=True,
+            text=True,
+            check=True,
+        )
+        gateway_ip = result.stdout.strip()
+        click.secho(f"Docker host IP (bridge gateway): {gateway_ip}")
+        return gateway_ip
+    except subprocess.CalledProcessError as e:
+        click.secho("Failed to inspect Docker network:")
+        abort(str(e))
+
+
+def _merge_odoo_dockerfile(config):
     """
     If customs contains dockerfile, then append their execution
     in the main dockerfile
@@ -1076,15 +1286,21 @@ def _merge_odoo_dockerfile(config, include_src):
         if "odoo/images/odoo" in dockerfile:
             shutil.copy(dockerfile, config.files["odoo_docker_file"])
             dockerfile1 = dockerfile
-            service["build"]["dockerfile"] = str(config.files["odoo_docker_file"])
+            service["build"]["dockerfile"] = str(
+                config.files["odoo_docker_file"]
+            )
         del dockerfile
     content = yaml.dump(content, default_flow_style=False)
     config.files["docker_compose"].write_text(content)
 
     # copy dockerfile to new location
     if dockerfile1:
-        click.secho(f"Copying {dockerfile1} to {config.files['odoo_docker_file']}")
-        config.files["odoo_docker_file"].write_text(Path(dockerfile1).read_text())
+        click.secho(
+            f"Copying {dockerfile1} to {config.files['odoo_docker_file']}"
+        )
+        config.files["odoo_docker_file"].write_text(
+            Path(dockerfile1).read_text()
+        )
         for file in bashfind(config.WORKING_DIR, "Dockerfile.appendix"):
             dir = file.parent / f"Dockerfile.appendix.dir"
             appendix = file.read_text()
@@ -1094,7 +1310,9 @@ def _merge_odoo_dockerfile(config, include_src):
             file.write_text(content)
             # probably a dir?
             if dir.exists():
-                dest = config.dirs['images'] / 'odoo' / "Dockerfile.appendix.dir"
+                dest = (
+                    config.dirs["images"] / "odoo" / "Dockerfile.appendix.dir"
+                )
                 if dest.exists() and dest.is_file():
                     dest.unlink()
                 dest.mkdir(parents=True, exist_ok=True)
@@ -1103,7 +1321,6 @@ def _merge_odoo_dockerfile(config, include_src):
                         shutil.copy(part, dest)
                     else:
                         shutil.copytree(part, dest / part.name)
-        
 
         # append common docker config
         odoo_docker_file = config.files["odoo_docker_file"]
@@ -1113,7 +1330,7 @@ def _merge_odoo_dockerfile(config, include_src):
         )
 
         # include source code
-        if include_src:
+        if not config.SRC_EXTRA:
             append_odoo_src(config, odoo_docker_file)
 
     else:
@@ -1122,7 +1339,15 @@ def _merge_odoo_dockerfile(config, include_src):
 
 
 def append_odoo_src(config, path):
-    content = "ADD {src} /opt/src"
+    linkdir = config.dirs["images"] / "odoo" / "src" / config.project_name
+    linkdir.parent.mkdir(parents=True, exist_ok=True)
+    # Remove existing symlink or file if necessary
+    sync_folder(config.HOST_CUSTOMS_DIR, linkdir, excludes=[".git"])
+    content = f"""
+RUN mkdir /opt/src
+COPY ./src/{config.project_name} /opt/src
+RUN find /opt/src -name .git -type d -exec rm -rf {{}} \\;
+    """
     path.write_text(path.read_text() + "\n" + content)
 
 
@@ -1145,7 +1370,9 @@ def _complete_setting_name(ctx, param, incomplete):
 
     res = set([x["name"].strip() for x in params])
     if incomplete:
-        res = list(filter(lambda x: x.lower().startswith(incomplete.lower()), res))
+        res = list(
+            filter(lambda x: x.lower().startswith(incomplete.lower()), res)
+        )
     return sorted(res)
 
 
@@ -1182,7 +1409,9 @@ def _complete_services(ctx, param, incomplete):
 
     if incomplete:
         services = list(
-            filter(lambda x: x.lower().startswith(incomplete.lower()), services)
+            filter(
+                lambda x: x.lower().startswith(incomplete.lower()), services
+            )
         )
     return services
 
@@ -1195,8 +1424,7 @@ def _complete_services(ctx, param, incomplete):
 @click.option("--file", required=False, default="")
 @click.option("-R", "--no-reload", is_flag=True)
 def docker_service(ctx, config, name, value, no_reload, file):
-    from .myconfigparser import MyConfigParser
-    import yaml
+    pass
 
     if not value and not file:
         content = _parse_yaml(__read_file(config.files["docker_compose"]))
@@ -1243,7 +1471,9 @@ def queuejob_channels(ctx, config, name, amount):
     channels = content.split(",")
 
     if name:
-        channels = list(filter(lambda x: x.strip().split(":")[0] != name, channels))
+        channels = list(
+            filter(lambda x: x.strip().split(":")[0] != name, channels)
+        )
         channels.append(f"{name}:{amount}")
         file.write_text(",".join(channels))
         click.secho("Now restart the queuejob container", fg="yellow")
@@ -1252,10 +1482,45 @@ def queuejob_channels(ctx, config, name, amount):
 
 
 def before_reload(config):
-    customs_dir = Path('.')
-    file = customs_dir / 'devscripts' / "before_reload.py"
+    customs_dir = Path(".")
+    file = customs_dir / "devscripts" / "before_reload.py"
     if not file.exists():
         return
     subprocess.check_call(["python3", file])
 
+
+def get_docker_host_ip():
+    try:
+        result = subprocess.run(
+            [
+                "docker",
+                "network",
+                "inspect",
+                "bridge",
+                "--format",
+                "{{(index .IPAM.Config 0).Gateway}}",
+            ],
+            capture_output=True,
+            text=True,
+            check=True,
+        )
+        gateway_ip = result.stdout.strip()
+        click.secho(f"Docker host IP (bridge gateway): {gateway_ip}")
+        return gateway_ip
+    except subprocess.CalledProcessError as e:
+        click.secho("Failed to inspect Docker network:")
+        abort(str(e))
+
+
+def get_host_info():
+    hostname = socket.gethostname()
+    fqdn = socket.getfqdn()
+    try:
+        local_ip = socket.gethostbyname(hostname)
+    except socket.gaierror:
+        local_ip = "Unknown"
+
+    return hostname, fqdn, local_ip
+
+
 Commands.register(do_reload, "reload")
