@@ -1,6 +1,5 @@
 import time
 import click
-from subprocess import Popen, PIPE
 
 import re
 import os
@@ -9,12 +8,12 @@ from .lib_clickhelpers import AliasedGroup
 from .tools import execute_script
 from .tools import force_input_hostname
 import subprocess
-import json
-from pathlib import Path
 from .tools import abort
 from .tools import ensure_project_name
 from .tools import print_prod_env
 
+APT_CACHER_CONTAINER_NAME = "apt-cacher"
+
 
 @cli.group(cls=AliasedGroup)
 @pass_config
@@ -108,16 +107,22 @@ def remove_volumes(ctx, config, dry_run):
                 for group in re.findall(r"(\[[^\]]*])", output):
                     container_id = group[1:-1]
                     subprocess.run(["docker", "kill", container_id])
-                    subprocess.check_call(["docker", "rm", "-fv", container_id])
+                    subprocess.check_call(
+                        ["docker", "rm", "-fv", container_id]
+                    )
                     counter = 0
                     while counter < 5:
                         try:
                             output = subprocess.check_output(
-                                ["docker", "volume", "rm", "-f", vol], encoding="utf8"
+                                ["docker", "volume", "rm", "-f", vol],
+                                encoding="utf8",
                             )
                             break
                         except:
-                            click.secho(f"Removing the volume {vol} failed - waiting and retrying.", fg="red")
+                            click.secho(
+                                f"Removing the volume {vol} failed - waiting and retrying.",
+                                fg="red",
+                            )
                             time.sleep(2)
                             counter += 1
 
@@ -177,7 +182,9 @@ def up(ctx, config, machines, daemon):
 
     lib_up(ctx, config, machines, daemon, remove_orphans=True)
     execute_script(
-        config, config.files["after_up_script"], "Possible after up script here:"
+        config,
+        config.files["after_up_script"],
+        "Possible after up script here:",
     )
     if daemon:
         _status(config)
@@ -194,6 +201,7 @@ def down(ctx, config, machines, volumes, remove_orphans, postgres_volume):
     ensure_project_name(config)
     from .lib_control_with_docker import down as lib_down
     from .lib_db_snapshots_docker_zfs import NotZFS
+
     if not config.devmode:
         if not config.force:
             abort("Please provide force option on non live systems")
@@ -273,6 +281,228 @@ def attach(ctx, config, machine):
     lib_attach(ctx, config, machine)
 
 
+def delete_gpg_files():
+    start_apt_cacher()
+    cmd = "find /var/cache/apt-cacher-ng/ -type f \( -name '*InRelease' -o -name '*Release.gpg' \)"
+    subprocess.run(
+        ["docker", "exec", APT_CACHER_CONTAINER_NAME, "sh", "-c", cmd]
+    )
+
+
+def start_apt_cacher():
+    container_name = APT_CACHER_CONTAINER_NAME
+    image_name = "sameersbn/apt-cacher-ng:latest"
+    network = "aptcache-net"  # necessary so name resolution works
+    port_mapping = "3142:3142"
+
+    def setup_options():
+        ancg_conf = (
+            subprocess.run(
+                [
+                    "docker",
+                    "exec",
+                    container_name,
+                    "cat",
+                    "/etc/apt-cacher-ng/acng.conf",
+                ],
+                capture_output=True,
+                text=True,
+                check=True,
+            )
+            .stdout.strip()
+            .splitlines()
+        )
+        ancg_conf = [
+            line.strip()
+            for line in ancg_conf
+            if line.strip() and not line.startswith("#")
+        ]
+        print("\n".join(ancg_conf))
+        if not any("ExThreshold: 0" in line for line in ancg_conf) or not any(
+            "DlMaxRetries: 5" in line for line in ancg_conf
+        ):
+            for option in [
+                "ExThreshold: 0",
+                "PassThroughPattern: .*Release|.*Packages|.*Sources",
+                "DlMaxRetries: 5",
+            ]:
+                click.secho(f"Adding option: {option}", fg="yellow")
+                subprocess.run(
+                    [
+                        "docker",
+                        "exec",
+                        container_name,
+                        "sh",
+                        "-c",
+                        f"echo '{option}' >> /etc/apt-cacher-ng/acng.conf",
+                    ]
+                )
+            subprocess.run(["docker", "restart", container_name])
+
+    # Check if container is already running
+    result = subprocess.run(
+        ["docker", "ps", "-q", "-f", f"name={container_name}"],
+        capture_output=True,
+        text=True,
+    )
+    if result.stdout.strip():
+        setup_options()
+        click.secho(f"Container '{container_name}' is already running.")
+        return
+
+    create_network(network)
+
+    # If not running, start it
+    result = subprocess.run(
+        ["docker", "ps", "-q", "-a", "-f", f"name={container_name}"],
+        capture_output=True,
+        text=True,
+    ).stdout.strip()
+    if result:
+        cmd = [
+            "docker",
+            "start",
+            result,
+        ]
+    else:
+        cmd = [
+            "docker",
+            "run",
+            "-d",
+            "--name",
+            container_name,
+            "--network",
+            network,
+            "-p",
+            port_mapping,
+            image_name,
+        ]
+    click.secho(f"Starting container '{container_name}'...", fg="blue")
+
+    try:
+        subprocess.run(cmd, check=True)
+        click.secho(
+            f"Container '{container_name}' started on port 3142.", fg="green"
+        )
+    except subprocess.CalledProcessError as e:
+        abort(str(e))
+
+    setup_options()
+
+
+def create_network(name="aptcache-net"):
+    # Check if the network already exists
+    result = subprocess.run(
+        [
+            "docker",
+            "network",
+            "ls",
+            "--filter",
+            f"name=^{name}$",
+            "--format",
+            "{{.Name}}",
+        ],
+        capture_output=True,
+        text=True,
+    )
+
+    if name in result.stdout.splitlines():
+        click.secho(f"Network '{name}' already exists.", fg="green")
+        return
+
+    # Create the network
+    click.secho(f"Creating Docker network '{name}'...", fg="yellow")
+    try:
+        subprocess.run(["docker", "network", "create", name], check=True)
+        click.secho(f"Network '{name}' created successfully.", fg="green")
+    except subprocess.CalledProcessError as e:
+        abort(str(e))
+
+
+@docker.command()
+@pass_config
+@click.pass_context
+def clear_apt_gpgfiles(ctx, config):
+    delete_gpg_files()
+    click.secho("Deleted GPG files in apt-cacher-ng cache", fg="green")
+
+
+@docker.command(help="ls /var/cache/apt/apt-cacher-ng")
+@click.argument("cache", nargs=-1)
+@pass_config
+@click.pass_context
+def clear_apt_cache(ctx, config, cache):
+    # Check if container is already running
+    result = subprocess.run(
+        ["docker", "ps", "-q", "-f", f"name={APT_CACHER_CONTAINER_NAME}"],
+        capture_output=True,
+        text=True,
+    )
+    containerid = result.stdout.strip()
+    if not containerid:
+        click.secho(f"Container '{APT_CACHER_CONTAINER_NAME}' is not running.")
+        return
+    if not cache:
+        result = subprocess.run(
+            [
+                "docker",
+                "exec",
+                "-it",
+                containerid,
+                "find",
+                "/var/cache/apt-cacher-ng/",
+                "-maxdepth",
+                "1",
+                "-type",
+                "d",
+                "-printf",
+                "%f\n",
+            ],
+            capture_output=True,
+            text=True,
+        )
+        options = []
+        for line in result.stdout.splitlines()[1:]:
+            if line.strip() in ["_xstore", "var"]:
+                continue
+            options.append(line)
+        import inquirer
+
+        todelete = inquirer.prompt(
+            [
+                inquirer.List(
+                    "filename", "Choose cache to delete", choices=options
+                )
+            ]
+        )["filename"]
+        if not todelete:
+            abort("No cache selected, aborting.")
+        todelete = [todelete]
+
+    else:
+        todelete = cache
+    for cache in todelete:
+        subprocess.run(
+            [
+                "docker",
+                "exec",
+                "-it",
+                containerid,
+                "rm",
+                "-Rf",
+                "/var/cache/apt-cacher-ng/{cache}",
+            ],
+            capture_output=True,
+            text=True,
+        )
+        click.secho(
+            f"Removed cache {cache} from {APT_CACHER_CONTAINER_NAME}",
+            fg="green",
+        )
+    click.secho("Stopping apt proxy...", fg="green")
+    subprocess.run(["docker", "kill", containerid])
+
+
 @docker.command()
 @click.argument("machines", nargs=-1)
 @click.option("--no-cache", is_flag=True)
@@ -280,23 +510,26 @@ def attach(ctx, config, machine):
 @click.option("--push", is_flag=True)
 @click.option("-p", "--plain", is_flag=True)
 @click.option("-s", "--include-source", is_flag=True)
-@click.option("-rm", "--remove", is_flag=True)
 @pass_config
 @click.pass_context
-def build(ctx, config, machines, pull, no_cache, push, plain, include_source, remove):
+def build(ctx, config, machines, pull, no_cache, push, plain, include_source):
     import yaml
+
+    start_apt_cacher()
+
     ensure_project_name(config)
     if plain:
         os.environ["BUILDKIT_PROGRESS"] = "plain"
     from .lib_control_with_docker import build as lib_build
+
     if not machines:
         compose = yaml.safe_load(config.files["docker_compose"].read_text())
         machines = []
-        for service in compose['services']:
-            if not compose['services'][service].get('build', {}).get('imgage'):
+        for service in compose["services"]:
+            if not compose["services"][service].get("build", {}).get("imgage"):
                 machines.append(service)
 
-    lib_build(ctx, config, machines, pull, no_cache, push, include_source, remove)
+    lib_build(ctx, config, machines, pull, no_cache, push, include_source)
 
 
 @docker.command()
@@ -327,7 +560,9 @@ def run(ctx, config, machine, detached, name, args, **kwparams):
     ensure_project_name(config)
     from .lib_control_with_docker import run as lib_run
 
-    lib_run(ctx, config, machine, args, detached=detached, name=name, **kwparams)
+    lib_run(
+        ctx, config, machine, args, detached=detached, name=name, **kwparams
+    )
 
 
 @cli.command()
@@ -405,7 +640,9 @@ def _get_project_volumes(config):
         x for x in system_volumes if x.startswith(config.project_name + "_")
     ]
 
-    full_volume_names = list(filter(lambda x: x in system_volumes, full_volume_names))
+    full_volume_names = list(
+        filter(lambda x: x in system_volumes, full_volume_names)
+    )
     return full_volume_names
 
 
@@ -433,7 +670,6 @@ def show_volumes(config, filter):
 @pass_config
 @click.pass_context
 def transfer_volume_content(context, config, show_all, filter, no_backup):
-    import shutil
     import inquirer
     from pathlib import Path
     from .lib_control_with_docker import _get_volume_size
@@ -456,14 +692,20 @@ def transfer_volume_content(context, config, show_all, filter, no_backup):
 
     if show_all:
         volumes = list(map(add_size, volumes))
-        volumes_filtered_to_project = [x for x in volumes if config.project_name in x]
+        volumes_filtered_to_project = [
+            x for x in volumes if config.project_name in x
+        ]
 
         volumes1 = volumes
         volumes2 = volumes_filtered_to_project
 
     else:
-        volumes_filtered_to_project = [x for x in volumes if config.project_name in x]
-        volumes_filtered_to_project = list(map(add_size, volumes_filtered_to_project))
+        volumes_filtered_to_project = [
+            x for x in volumes if config.project_name in x
+        ]
+        volumes_filtered_to_project = list(
+            map(add_size, volumes_filtered_to_project)
+        )
 
         volumes1 = volumes_filtered_to_project
         volumes2 = volumes_filtered_to_project
@@ -484,7 +726,9 @@ def transfer_volume_content(context, config, show_all, filter, no_backup):
     questions = [
         inquirer.List(
             "volume",
-            message="Select Destination:".format(config.customs, config.dbname),
+            message="Select Destination:".format(
+                config.customs, config.dbname
+            ),
             choices=volumes2,
         ),
     ]
@@ -498,7 +742,9 @@ def transfer_volume_content(context, config, show_all, filter, no_backup):
     tasks.append(f"rsync -ar --delete-after {source.name} to {dest.name}")
     for i, task in enumerate(tasks):
         click.secho(f"{i}. {task}")
-    answer = inquirer.prompt([inquirer.Confirm("continue", message=("Continue?"))])
+    answer = inquirer.prompt(
+        [inquirer.Confirm("continue", message=("Continue?"))]
+    )
     if not answer["continue"]:
         return
     Commands.invoke(context, "down")
@@ -552,9 +798,18 @@ def docker_sizes(context, config):
             yaml.safe_load(output)["services"].keys(),
         )
     )
-    out = subprocess.check_output(
-        ["docker", "images", "--format", "{{.Repository}}:{{.Tag}}\tSize: {{.Size}}"]
-    ).decode('utf8').splitlines()
+    out = (
+        subprocess.check_output(
+            [
+                "docker",
+                "images",
+                "--format",
+                "{{.Repository}}:{{.Tag}}\tSize: {{.Size}}",
+            ]
+        )
+        .decode("utf8")
+        .splitlines()
+    )
     sizes = {}
     for line in out:
         name, size = line.split(":", 1)
