@@ -1,18 +1,9 @@
-import platform
 import sys
 import shutil
-import hashlib
 import os
-import tempfile
 import click
 from .tools import __dcrun
-from .tools import __assert_file_exists
-from .tools import __safe_filename
-from .tools import __read_file
-from .tools import __write_file
-from .tools import __append_line
 from .tools import _askcontinue
-from .tools import __get_odoo_commit
 from .tools import _is_container_running
 from .tools import _get_bash_for_machine
 from .tools import __cmd_interactive
@@ -21,7 +12,6 @@ from .tools import _wait_postgres
 from .tools import __replace_in_file
 from .tools import _wait_for_port
 from .tools import __dcexec
-from .tools import _get_machines
 from .tools import __dc
 from .tools import _get_host_ip
 from .tools import __needs_docker
@@ -96,17 +86,22 @@ def dev(ctx, config, build, kill):
     proxy_port = myconfig["PROXY_PORT"]
     roundcube_port = myconfig["ROUNDCUBE_PORT"]
     click.secho(
-        "Proxy Port: http://{}:{}".format(ip, proxy_port), fg="green", bold=True
+        "Proxy Port: http://{}:{}".format(ip, proxy_port),
+        fg="green",
+        bold=True,
     )
     click.secho(
-        "Mailclient : http://{}:{}".format(ip, roundcube_port), fg="green", bold=True
+        "Mailclient : http://{}:{}".format(ip, roundcube_port),
+        fg="green",
+        bold=True,
     )
 
     # execute script
     ScriptFile = config.files["start-dev"]
     if not ScriptFile.exists():
         click.secho(
-            f"Info: you may provide a startup script here: {ScriptFile}", fg="yellow"
+            f"Info: you may provide a startup script here: {ScriptFile}",
+            fg="yellow",
         )
     else:
         FNULL = open(os.devnull, "w")
@@ -150,10 +145,15 @@ def do_kill(ctx, config, machines=[], brutal=False, profile="auto"):
             __dc(
                 config, ["stop", "-t", "20"] + safe_stop, profile=profile
             )  # persist data
-    if config.devmode:
-        __dc(config, ["kill"] + list(machines), profile=profile)
-    else:
-        __dc(config, ["stop", "-t", "2"] + list(machines), profile=profile)
+    try:
+        if config.devmode:
+            __dc(config, ["kill"] + list(machines), profile=profile)
+        else:
+            __dc(config, ["stop", "-t", "2"] + list(machines), profile=profile)
+    except subprocess.CalledProcessError as e:
+        pass
+        # was not possible to handle the not running error
+        # chat gpt also suggests to maximally handle no such container or container not running
 
 
 def force_kill(ctx, config, machine, profile="auto"):
@@ -175,8 +175,11 @@ def recreate(ctx, config, machines=[]):
     __dc(config, ["up", "--no-start", "--force-recreate"] + machines)
 
 
-def up(ctx, config, machines=[], daemon=False, remove_orphans=True, profile="auto"):
+def up(
+    ctx, config, machines=[], daemon=False, remove_orphans=True, profile="all"
+):
     machines = list(machines)
+    from .consts import resolve_profiles
 
     options = [
         # '--remove-orphans', # lost data with that; postgres volume suddenly new after rm?
@@ -187,12 +190,11 @@ def up(ctx, config, machines=[], daemon=False, remove_orphans=True, profile="aut
     if remove_orphans:
         options += ["--remove-orphans"]
     dc_options = []
-    if profile:
-        dc_options += ["--profile", profile]
-
     if not machines and config.run_postgres and daemon and config.USE_DOCKER:
         _start_postgres_before(config)
-    __dc(config, dc_options + ["up"] + options + machines)
+    for profile in resolve_profiles(profile):
+        dc_options2 = dc_options + ["--profile", profile]
+        __dc(config, dc_options2 + ["up"] + options + machines)
 
 
 def down(ctx, config, machines=[], volumes=False, remove_orphans=True):
@@ -222,11 +224,11 @@ def rebuild(ctx, config, machines=[]):
     build(ctx, config, machines=machines, no_cache=True)
 
 
-def restart(ctx, config, machines=[], profile="auto"):
+def restart(ctx, config, machines=[], profile="auto", brutal=True):
     machines = list(machines)
 
     # this is faster than docker restart: tested with normal project 6.75 seconds vs. 4.8 seconds
-    do_kill(ctx, config, machines=machines, profile=profile)
+    do_kill(ctx, config, machines=machines, profile=profile, brutal=brutal)
     up(ctx, config, machines=machines, daemon=True, profile=profile)
 
 
@@ -258,7 +260,6 @@ def build(
     no_cache=False,
     push=False,
     include_source=False,
-    remove=False,
 ):
     """
     no parameter all machines, first parameter machine name and passes other params; e.g. ./odoo build asterisk --no-cache"
@@ -266,8 +267,6 @@ def build(
     options = []
     if pull:
         options += ["--pull"]
-    if remove:
-        options += ["--force-rm"]
     if no_cache:
         options += ["--no-cache"]
         if "--pull" not in options:
@@ -279,11 +278,19 @@ def build(
     if include_source:
         raise NotImplementedError("Please implement include source.")
 
+    platform = subprocess.check_output(
+        ["/usr/bin/uname", "-m"], encoding="utf8"
+    ).strip()
+    # options += ["--platform", platform]
+
     __dc(
         config,
         ["build"] + options + list(machines),
         env={
             "ODOO_VERSION": config.odoo_version,  # at you developer: do not mismatch with build args
+            "DOCKER_DEFAULT_PLATFORM": f"linux/{platform}",
+            "DOCKER_BUILDKIT": "1",
+            "COMPOSE_BAKE": "true",
         },
     )
 
@@ -315,7 +322,9 @@ def debug(ctx, config, machine, ports, cmd=None):
         shutil.copy(filepath, dest)
         __replace_in_file(dest, "__PORT__", ports or "33284")
         __replace_in_file(dest, "${NAME}", machine)
-        __replace_in_file(dest, "${DOCKER_COMPOSE_VERSION}", config.YAML_VERSION)
+        __replace_in_file(
+            dest, "${DOCKER_COMPOSE_VERSION}", config.YAML_VERSION
+        )
 
         # TODO make configurable in machines
         PORT = str({"odoo": 8069, "odoo_debug": 8069}.get(machine, 80))
@@ -366,6 +375,11 @@ def shell(config, command="", queuejobs=False):
     cmd = [
         "run",
         "--rm",
+        "-it",
+        "-e",
+        "TERM=xterm-256color",
+        "-e",
+        "PYTHONUNBUFFERED=1",
         "odoo",
         "/odoolib/shell.py",
     ]
