@@ -18,6 +18,9 @@ from .tools import print_prod_env
 from .tools import _exists_db
 from .odoo_config import get_conn_autoclose
 from .tools import __dc
+from tqdm import tqdm
+
+SEP = "------------------------------------------------------------"
 
 
 @cli.group(cls=AliasedGroup)
@@ -27,7 +30,10 @@ def db(config):
     Database related actions.
     """
     click.echo(
-        (f"database-name: {config.dbname}, " f"in ram: {config.run_postgres_in_ram}")
+        (
+            f"database-name: {config.dbname}, "
+            f"in ram: {config.run_postgres_in_ram}"
+        )
     )
 
 
@@ -37,7 +43,9 @@ def db_health_check(config):
     conn = config.get_odoo_conn()
     click.secho((f"Connecting to {conn.host}:{conn.port}/{config.dbname}"))
     try:
-        _execute_sql(conn, "select * from pg_catalog.pg_tables;", fetchall=True)
+        _execute_sql(
+            conn, "select * from pg_catalog.pg_tables;", fetchall=True
+        )
     except Exception:  # pylint: disable=broad-except
         abort("Listing tables failed for connection to {conn.host}")
     else:
@@ -52,7 +60,9 @@ def drop_db(config, dbname):
         click.secho("Either DEVMODE or force required", fg="red")
         sys.exit(-1)
     conn = config.get_odoo_conn().clone(dbname="postgres")
-    _remove_postgres_connections(conn, sql_afterwards=f"drop database {dbname};")
+    _remove_postgres_connections(
+        conn, sql_afterwards=f"drop database {dbname};"
+    )
     click.echo(f"Database {dbname} dropped.")
 
 
@@ -62,7 +72,11 @@ def pgactivity(config):
     from .tools import DBConnection
 
     conn = DBConnection(
-        config.dbname, config.db_host, config.db_port, config.db_user, config.db_pwd
+        config.dbname,
+        config.db_host,
+        config.db_port,
+        config.db_user,
+        config.db_pwd,
     )
     __dcrun(
         config,
@@ -120,7 +134,9 @@ def pgcli(config, dbname, params, host, port, user, password):
 def psql(config, dbname, params, sql, non_interactive):
     dbname = dbname or config.dbname
     conn = config.get_odoo_conn(inside_container=True).clone(dbname=dbname)
-    return _psql(config, conn, params, sql=sql, interactive=not non_interactive)
+    return _psql(
+        config, conn, params, sql=sql, interactive=not non_interactive
+    )
 
 
 def _psql(
@@ -175,7 +191,13 @@ def _psql(
 
 
 def _pgcli(config, conn, params, use_docker_container=None):
-    _psql(config, conn, params, bin="pgcli", use_docker_container=use_docker_container)
+    _psql(
+        config,
+        conn,
+        params,
+        bin="pgcli",
+        use_docker_container=use_docker_container,
+    )
 
 
 def aggressive_drop_db(config, conn, dbname):
@@ -183,10 +205,13 @@ def aggressive_drop_db(config, conn, dbname):
         try:
             _dropdb(config, conn, dbname)
         except Exception as ex:
-            click.secho(f"Error at dropping db at attempt {i + 1}: {ex}", fg="red")
+            click.secho(
+                f"Error at dropping db at attempt {i + 1}: {ex}", fg="red"
+            )
             if config.run_postgres:
                 click.secho(
-                    f"Restarting postgres to remove any connections.", fg="yellow"
+                    f"Restarting postgres to remove any connections.",
+                    fg="yellow",
                 )
                 __dc(config, ["kill", "postgres"])
                 __dc(config, ["up", "-d", "postgres"])
@@ -207,6 +232,7 @@ def aggressive_drop_db(config, conn, dbname):
 @click.pass_context
 def reset_db(ctx, config, dbname, do_not_install_base, no_overwrite):
     import psycopg2
+
     collatec = True
     dbname = dbname or config.dbname
     if not dbname:
@@ -219,7 +245,9 @@ def reset_db(ctx, config, dbname, do_not_install_base, no_overwrite):
         try:
             with get_conn_autoclose() as cr:
                 if _is_db_initialized(cr):
-                    click.secho("Database already initialized. Skipping.", fg="yellow")
+                    click.secho(
+                        "Database already initialized. Skipping.", fg="yellow"
+                    )
                     return
         except Exception:
             abort(
@@ -237,7 +265,10 @@ def reset_db(ctx, config, dbname, do_not_install_base, no_overwrite):
         try:
             _execute_sql(conn, cmd, notransaction=True)
             break
-        except psycopg2.errors.DuplicateDatabase:
+        except (
+            psycopg2.errors.UniqueViolation,
+            psycopg2.errors.DuplicateDatabase,
+        ):
             aggressive_drop_db(config, conn, dbname)
 
     # since odoo version 12 "-i base -d <name>" is required
@@ -253,6 +284,7 @@ def reset_db(ctx, config, dbname, do_not_install_base, no_overwrite):
             no_update_module_list=True,
             non_interactive=True,
             no_outdated_modules=True,
+            no_scripts=True,
         )
 
 
@@ -265,7 +297,10 @@ def anonymize(ctx, config):
         sys.exit(-1)
 
     _make_sure_module_is_installed(
-        ctx, config, "anonymize", "https://github.com/marcwimmer/odoo-anonymize.git"
+        ctx,
+        config,
+        "anonymize",
+        "https://github.com/marcwimmer/odoo-anonymize.git",
     )
 
     Commands.invoke(
@@ -288,7 +323,10 @@ def cleardb(ctx, config, no_vacuum_full):
         sys.exit(-1)
 
     _make_sure_module_is_installed(
-        ctx, config, "cleardb", "https://github.com/marcwimmer/odoo-cleardb.git"
+        ctx,
+        config,
+        "cleardb",
+        "https://github.com/marcwimmer/odoo-cleardb.git",
     )
     str_no_vauum_full = "1" if no_vacuum_full else "0"
 
@@ -380,7 +418,8 @@ ORDER BY total_bytes DESC;
         rows = rows[:top]
     click.echo(
         tabulate(
-            rows, ["TABLE_NAME", "row_estimate", "total", "INDEX", "toast", "TABLE"]
+            rows,
+            ["TABLE_NAME", "row_estimate", "total", "INDEX", "toast", "TABLE"],
         )
     )
 
@@ -421,7 +460,8 @@ def excel(config, sql, file, base64):
         filepath = Path(os.getcwd()) / file
     else:
         filepath = Path(os.getcwd()) / (
-            f"{conn.dbname}_" f"{arrow.get().strftime('%Y-%m-%d%H-%M-%S')}.xlsx"
+            f"{conn.dbname}_"
+            f"{arrow.get().strftime('%Y-%m-%d%H-%M-%S')}.xlsx"
         )
 
     # Workbook() takes one, non-optional, argument
@@ -472,12 +512,16 @@ def pghba_conf_wide_open(config, no_scram):
 
         _execute_sql(conn, f"copy hba from '{pghba_conf}';")
         _execute_sql(
-            conn, ("delete from hba " "where lines like 'host%all%all%all%md5'")
+            conn,
+            ("delete from hba " "where lines like 'host%all%all%all%md5'"),
         )
         for method in ["trust", "scram", "md5"]:
             _execute_sql(
                 conn,
-                ("delete from hba " f"where lines like 'host%all%all%all%{method}'"),
+                (
+                    "delete from hba "
+                    f"where lines like 'host%all%all%all%{method}'"
+                ),
             )
 
         def trustline():
@@ -488,7 +532,9 @@ def pghba_conf_wide_open(config, no_scram):
                     trustline = "host all all all md5"
                 else:
                     trustline = "host all all all scram-sha-256"
-            _execute_sql(conn, (f"insert into hba(lines) values('{trustline}');"))
+            _execute_sql(
+                conn, (f"insert into hba(lines) values('{trustline}');")
+            )
 
         trustline()
 
@@ -521,9 +567,12 @@ def pghba_conf_wide_open(config, no_scram):
 
         _execute_sql(conn, f"copy hba from '{conf}' with (delimiter E'~');")
         _execute_sql(
-            conn, ("delete from hba " "where lines like '%password_encryption%'")
+            conn,
+            ("delete from hba " "where lines like '%password_encryption%'"),
+        )
+        _execute_sql(
+            conn, ("update hba set lines = replace(lines, '\t', ' ')")
         )
-        _execute_sql(conn, ("update hba set lines = replace(lines, '\t', ' ')"))
         if no_scram:
             _execute_sql(
                 conn,
@@ -550,5 +599,292 @@ def pghba_conf_wide_open(config, no_scram):
     adapt_postgres_conf()
 
 
+def shorten_string(s, max_length=30):
+    if len(s) <= max_length:
+        return s.ljust(30)
+    half = (max_length - 3) // 2
+    res = s[:half] + "..." + s[-half:]
+    res = res.ljust(max_length)
+    res = res[:max_length]
+    return res
+
+
+def compare(file1, file2, ignore_magic, skip_no_id_tables):
+    def parse(file):
+        records = {}
+        idx = 0
+        record = None
+        for line in file.splitlines() + [SEP]:
+            if line == SEP:
+                if record:
+                    if "id" in record:
+                        records[record["id"]] = record
+                    else:
+                        if skip_no_id_tables:
+                            return {}
+                        records[idx] = record
+                record = {}
+                continue
+            key, value = line.split(": ", 1)
+            key = key.strip()
+            value = value.strip()
+            record[key] = value
+            idx += 1
+        return records
+
+    data1 = parse(file1.read_text())
+    data2 = parse(file2.read_text())
+
+    result = {"count": 0, "missing_left": [], "missing_right": [], "diffs": []}
+    for id, r1 in data1.items():
+        r2 = data2.get(id, None)
+        if r2 is None:
+            result["missing_right"].append(r1)
+            result["count"] += 1
+            continue
+        inc_count = False
+        for key, value in r1.items():
+            if ignore_magic and key in ["create_date", "write_date"]:
+                continue
+            value2 = r2.get(key)
+            if value != value2:
+                if "<memory at" not in value:
+                    inc_count = True
+                    result["diffs"].append(
+                        (
+                            {
+                                "id": id,
+                                "field": key,
+                                "value1": value,
+                                "value2": value2,
+                            }
+                        )
+                    )
+        if inc_count:
+            result["count"] += 1
+    for id, r2 in data1.items():
+        r1 = data1.get(id)
+        if r1 is None:
+            result["missing_left"].append(r2)
+            result["count"] += 1
+    return result
+
+
+@db.command()
+@pass_config
+@click.argument("path1", required=True)
+@click.argument("path2", required=True)
+@click.option("-D", "--no_details", is_flag=True)
+@click.option("-i", "--include")
+@click.option("-s", "--skip")
+@click.option(
+    "-I",
+    "--ignore-magic",
+    is_flag=True,
+    help="Ignore magic columns like create_date, write_date",
+)
+@click.option(
+    "--include-no-id-tables",
+    is_flag=True,
+    help="Ignore rel tables without id column",
+)
+def dbcompare(
+    config,
+    path1,
+    path2,
+    ignore_magic,
+    include,
+    include_no_id_tables,
+    skip,
+    no_details,
+):
+    path1 = Path(path1)
+    path2 = Path(path2)
+    missing_files = []
+    diffs = {}
+    files2 = list(sorted(path2.glob("*.dat")))
+    include = list(
+        filter(bool, map(lambda x: x.strip(), (include or "").split(",")))
+    )
+    skip = list(
+        filter(bool, map(lambda x: x.strip(), (skip or "").split(",")))
+    )
+    progress = tqdm(
+        total=len(files2),
+        bar_format="\033[95m{l_bar}{bar}\033[0m {n_fmt}/{total_fmt} [{elapsed}<{remaining}]",
+        dynamic_ncols=True,
+    )
+
+    def includeme(file):
+        if skip:
+            if file.stem in skip:
+                return False
+        if not include:
+            return True
+        return file.stem in include
+
+    for i, file in enumerate(files2):
+        if not includeme(file):
+            continue
+        progress.n = i
+        progress.set_description(shorten_string(file.name, 30))
+        progress.refresh()
+        file = Path(file)
+        file1 = list(path1.glob(file.name))
+        if not file1:
+            missing_files.append(file)
+            continue
+        diffs[file] = compare(
+            file,
+            Path(file1[0]),
+            ignore_magic=ignore_magic,
+            skip_no_id_tables=not include_no_id_tables,
+        )
+    for file in path1.glob("*.dat"):
+        if not includeme(file):
+            continue
+        file2 = list(path2.glob(file.name))
+        if not file2:
+            missing_files.append(file)
+            continue
+
+    if not no_details:
+        for filename, value in diffs.items():
+            if not value["diffs"]:
+                continue
+            click.secho(f"Diff for same id: {filename.name}", fg="green")
+            for diff in value["diffs"]:
+                click.secho(
+                    f"{diff['field']}  {diff['value1']} <----> {diff['value2']}"
+                )
+    else:
+        for filename, value in diffs.items():
+            count = len(value["diffs"])
+            if count:
+                click.secho(f"Diffs in {filename}: {count}")
+
+    def print_missing(missings):
+        for missing in missings:
+            click.secho(f"{missing}")
+
+    for missing in missing_files:
+        click.secho(f"Missing: {missing}")
+    for key, value in diffs.items():
+        count_diffs = value["count"]
+        if not count_diffs:
+            continue
+        if value["missing_left"]:
+            click.secho(f"Only in {key}", fg="green")
+            print_missing(value["missing_left"])
+        if value["missing_right"]:
+            click.secho(f"Only in {key}", fg="green")
+            print_missing(value["missing_right"])
+
+
+@db.command()
+@pass_config
+@click.argument("output", required=True)
+@click.option("-f", "--table-filter")
+@click.option("-l", "--limit")
+@click.option("-s", "--skip", help="Provide table names (comma separated)")
+@click.option("-i", "--include", help="Actively filter on this")
+def filesnapshot(config, table_filter, output, limit, skip, include):
+    conn = config.get_odoo_conn()
+    skip = list(
+        filter(bool, map(lambda x: x.strip(), (skip or "").split(",")))
+    )
+    include = list(
+        filter(bool, map(lambda x: x.strip(), (include or "").split(",")))
+    )
+
+    if not skip:
+        skip = [
+            "queue_job",
+            "datapolice_increment",
+            "ir_attachment",
+            "mail_message",
+            "mail_followers",
+            "mail_tracking_value",
+        ]
+    all_names = [
+        x[0]
+        for x in _execute_sql(
+            conn,
+            f"select table_name from information_schema.tables where table_schema = 'public' and table_type='BASE TABLE';",
+            fetchall=True,
+        )
+    ]
+
+    if table_filter:
+        table_filter_splitted = list(
+            map(lambda x: x.strip(), table_filter.split(","))
+        )
+        all_names = list(
+            filter(
+                lambda name: any(x in name for x in table_filter_splitted),
+                all_names,
+            )
+        )
+
+    progress = tqdm(
+        total=len(all_names),
+        bar_format="\033[95m{l_bar}{bar}\033[0m {n_fmt}/{total_fmt} [{elapsed}<{remaining}]",
+        dynamic_ncols=True,
+    )
+
+    root_path = Path(output)
+    root_path.mkdir(exist_ok=True, parents=True)
+    count_records = 0
+    for i, item in enumerate(sorted(all_names)):
+        progress.n = i
+        progress.set_description(shorten_string(item, 30))
+        # progress.refresh()
+        if item in skip and not include:
+            continue
+        if include and item not in include:
+            continue
+        columns, rows = _execute_sql(
+            conn,
+            f"select * from {item} limit 0",
+            fetchall=True,
+            return_columns=True,
+        )
+
+        sql = f"select * from {item}"
+        if "id" in columns:
+            sql = sql + " order by id desc"
+        if limit:
+            sql = sql + f" limit {limit}"
+        columns, rows = _execute_sql(
+            conn, sql, fetchall=True, return_columns=True
+        )
+        columns_sorted = list(sorted(columns))
+        output_file = root_path / f"{item}.dat"
+        output_file.write_text("")
+
+        max_column_length = max(map(len, columns))
+
+        def iterate():
+            yield from rows
+            # if 'id' in columns:
+            #     idx = columns.index('id')
+            #     yield from sorted(rows, key=lambda x: x[idx], reverse=True)
+            # else:
+            #     yield from rows
+
+        with open(output_file, "w") as f:
+            for irow, row in enumerate(iterate()):
+                f.write(SEP + "\n")
+                for column in columns_sorted:
+                    icolumn = columns.index(column)
+                    collabel = columns[icolumn].ljust(max_column_length, " ")
+                    value = (
+                        str(row[icolumn]).replace("\n", " ").replace("\r", " ")
+                    )
+                    f.write(f"{collabel}: {value}\n")
+                count_records += 1
+    click.secho(f"Exported {count_records} records to {root_path}")
+
+
 Commands.register(reset_db, "reset-db")
 Commands.register(pghba_conf_wide_open, "pghba_conf_wide_open")
