@@ -1,12 +1,10 @@
 """
-To work the volumes folder must be a child of a zfs filesystem:
+To be able to do snapshots: the volumes folder must be a child of a zfs filesystem:
 zfs create zfs_pool1/docker
 zfs create zfs_pool1/docker/volumes
 
-set /etc/odoo/settings ZFS_PATH_VOLUMES=zfs_pool1/docker/volumes  then
 
-
-zfs list -o name,usedbychildren     # 
+zfs list -o name,usedbychildren     #
 docker/vlsa039t563j1yi1k4hum45or                                                0B
 docker/volumes                                                                100K
 docker/volumes/t1                                                               0B
@@ -17,8 +15,7 @@ better: zfs get -H -o value usedbychildren docker/volumes  mit -1 oder nicht
 """
 
 import inquirer
-import os
-import json
+import time
 from .tools import abort
 from operator import itemgetter
 import subprocess
@@ -29,11 +26,9 @@ import tempfile
 import click
 from .tools import __dc
 from .tools import search_env_path, __get_postgres_volume_name
-from .cli import cli, pass_config
-from .lib_clickhelpers import AliasedGroup
 from pathlib import Path
 from .tools import abort
-from .tools import get_volume_fullpath, verbose
+from .tools import get_volume_fullpath
 
 HOWTO_PREPARE = """
 
@@ -96,6 +91,59 @@ def _get_path(config):
 CACHE_ZFS_PATH = None
 
 
+def check_correct_zfs_setup(config):
+    # get mountpoint of zfs pool or zfs
+    PATH = docker_volume_path()
+    if not __is_zfs_fs(PATH):
+        return
+    zfspool_or_zfsvolume = _get_zfs_pool_or_zfs_parent(PATH)
+    zfspool_mountpath = (
+        subprocess.check_output(
+            [
+                "zfs",
+                "get",
+                "-H",
+                "-o",
+                "value",
+                "mountpoint",
+                zfspool_or_zfsvolume,
+            ],
+            encoding="utf8",
+        )
+        .splitlines()[-1]
+        .strip()
+    )
+    relative_path_to_mountpath = Path(PATH).relative_to(zfspool_mountpath)
+
+    if relative_path_to_mountpath.parts:
+        abort(
+            "\nZFS Misconfiguration detected\n------------------------------------\n"
+            "There mustn't be any relative path "
+            f'between the zfs pool or parent zfs "{zfspool_or_zfsvolume}" \nand the mountpoint '
+            f"{PATH}/somevolume. \n"
+            f"To solve this, create a zfs mounted at {PATH} e.g. with \n"
+            "zfs create pool1/docker_volumes \n"
+            "zfs set compression=on pool1/docker_volumes \n"
+            f"zfs set mountpoint={PATH} pool1/docker_volumes \n"
+            "\n\n"
+            "And restart docker."
+        )
+
+
+def _get_zfs_pool_or_zfs_parent(path):
+    try:
+        findmnt = subprocess.check_output(
+            ["findmnt", "--target", path, "--output", "SOURCE"],
+            encoding="utf8",
+        ).splitlines()
+    except subprocess.CalledProcessError:
+        abort(f"No zfs pool found for {path}")
+    if not findmnt:
+        abort(f"No zfs pool found for {path}")
+    zfspool = findmnt[1].strip()
+    return zfspool
+
+
 def _get_zfs_path(config):
     """
     takes the postgresname and translates:
@@ -108,25 +156,20 @@ def _get_zfs_path(config):
     if CACHE_ZFS_PATH is None:
         PATH = docker_volume_path()
         postgresname = __get_postgres_volume_name(config)
-        try:
-            findmnt = subprocess.check_output(
-                ["findmnt", "--target", PATH, "--output", "SOURCE"], encoding="utf8"
-            ).splitlines()
-        except subprocess.CalledProcessError:
-            abort(f"No zfs pool found for {PATH}")
-        if not findmnt:
-            abort(f"No zfs pool found for {PATH}")
+        zfspool = _get_zfs_pool_or_zfs_parent(PATH)
 
         fstype = subprocess.check_output(
-            ["findmnt", "--target", PATH, "--output", "FSTYPE"], encoding="utf8"
+            ["findmnt", "--target", PATH, "--output", "FSTYPE"],
+            encoding="utf8",
         ).splitlines()
         if fstype[1].strip() != "zfs":
             abort(f"No zfs pool found for {PATH}")
 
+        check_correct_zfs_setup(config)
+
         # TARGET                  SOURCE        FSTYPE OPTIONS
         # /var/lib/docker/volumes dockervolumes zfs    rw,relatime,xattr,noacl,casesensitive
-        zfsfolder = findmnt[1].strip()
-        CACHE_ZFS_PATH = str(Path(zfsfolder) / postgresname)
+        CACHE_ZFS_PATH = str(Path(zfspool) / postgresname)
     return CACHE_ZFS_PATH
 
 
@@ -157,7 +200,10 @@ def _get_possible_snapshot_paths(config):
     ]
 
     def matches(path):
-        return "/" + postgresvolume + "." in path or "/" + postgresvolume + "@" in path
+        return (
+            "/" + postgresvolume + "." in path
+            or "/" + postgresvolume + "@" in path
+        )
 
     snaps = list(filter(matches, snaps))
     yield from snaps
@@ -209,7 +255,7 @@ def assert_environment(config):
 
 def _turn_into_subvolume(config):
     """
-    Makes a zfs pool out of a path.
+    Makes a zfs volume out of a path.
     """
     if config.NAMED_ODOO_POSTGRES_VOLUME:
         abort("Not compatible with NAMED_ODOO_POSTGRES_VOLUME by now.")
@@ -224,7 +270,9 @@ def _turn_into_subvolume(config):
     if filename.exists():
         raise Exception(f"Path {filename} should not exist.")
     if not fullpath.exists():
-        abort(f"{fullpath} does not exist. Did you start postgres? (odoo up -d)")
+        abort(
+            f"{fullpath} does not exist. Did you start postgres? (odoo up -d)"
+        )
 
     shutil.move(fullpath, filename)
     try:
@@ -257,13 +305,16 @@ def make_snapshot(ctx, config, name):
             answer = inquirer.prompt(
                 [
                     inquirer.Confirm(
-                        "continue", message=("Snapshot already exists - overwrite?")
+                        "continue",
+                        message=("Snapshot already exists - overwrite?"),
                     )
                 ]
             )
             if not answer["continue"]:
                 sys.exit(-1)
-        subprocess.check_call(["sudo", zfs, "destroy", snapshot[0]["fullpath"]])
+        subprocess.check_call(
+            ["sudo", zfs, "destroy", snapshot[0]["fullpath"]]
+        )
 
     assert " " not in name
     fullpath = _get_zfs_path(config) + "@" + name
@@ -289,7 +340,8 @@ def _try_umount(config):
         )
     except subprocess.CalledProcessError:
         click.secho(
-            f"Could not umount {zfs_full_path}. Perhaps not a problem.", fg="yellow"
+            f"Could not umount {zfs_full_path}. Perhaps not a problem.",
+            fg="yellow",
         )
 
 
@@ -308,7 +360,9 @@ def restore(config, name):
     snapshot = snapshot[0]
     zfs_full_path = _get_zfs_path(config)
     snapshots_of_volume = [
-        x for x in snapshots if x["fullpath"].split("@")[0].startswith(zfs_full_path)
+        x
+        for x in snapshots
+        if x["fullpath"].split("@")[0].startswith(zfs_full_path)
     ]
     try:
         index = list(map(lambda x: x["name"], snapshots_of_volume)).index(name)
@@ -319,19 +373,22 @@ def restore(config, name):
     full_next_path = _get_next_snapshotpath(config)
     _try_umount(config)
     if _is_zfs_path(zfs_full_path):
-        subprocess.check_call(["sudo", zfs, "rename", zfs_full_path, full_next_path])
+        subprocess.check_call(
+            ["sudo", zfs, "rename", zfs_full_path, full_next_path]
+        )
     snap_name = snapshot["fullpath"].split("@")[-1]
     snapshot_path = _get_zfs_path_for_snap_name(config, snap_name)
-    subprocess.check_call(
-        [
-            "sudo",
-            zfs,
-            "clone",
-            snapshot_path,
-            zfs_full_path,
-        ]
-    )
     __dc(config, ["rm", "-f"] + ["postgres"])
+    cmd = [
+        "sudo",
+        zfs,
+        "clone",
+        snapshot_path,
+        zfs_full_path,
+    ]
+    subprocess.check_call(cmd)
+    click.secho(f"Restore command:")
+    click.secho(" ".join(map(str, cmd)), fg="yellow")
     __dc(config, ["up", "-d"] + ["postgres"])
 
 
@@ -353,7 +410,9 @@ def remove(config, snapshot):
         snapshot = snapshots[0]
     if snapshot["fullpath"] in map(itemgetter("fullpath"), snapshots):
         _try_umount(config)
-        subprocess.check_call(["sudo", zfs, "destroy", "-R", snapshot["fullpath"]])
+        subprocess.check_call(
+            ["sudo", zfs, "destroy", "-R", snapshot["fullpath"]]
+        )
         remount(config)
 
 
@@ -362,7 +421,9 @@ def remove_volume(config):
     umount = search_env_path("umount")
     for path in _get_possible_snapshot_paths(config):
         try:
-            subprocess.check_call(["sudo", zfs, "set", "canmount=noauto", path])
+            subprocess.check_call(
+                ["sudo", zfs, "set", "canmount=noauto", path]
+            )
         except:
             click.secho(
                 "Failed to execute canmount=noauto, but perhaps not a problem. Trying to continue.",
@@ -373,14 +434,22 @@ def remove_volume(config):
         except:
             pass
 
-        fullpath = translate_poolPath_to_fullPath(Path(path).parent) / Path(path).name
+        fullpath = (
+            translate_poolPath_to_fullPath(Path(path).parent) / Path(path).name
+        )
         if fullpath.exists() or "@" in str(fullpath):
-            subprocess.check_call(
-                ["sudo", zfs, "destroy", "-R", path],
-                encoding="utf8",
-                stderr=subprocess.PIPE,
-                stdout=subprocess.PIPE,
-            )
+            try:
+                subprocess.check_call(
+                    ["sudo", zfs, "destroy", "-R", path],
+                    encoding="utf8",
+                    stderr=subprocess.PIPE,
+                    stdout=subprocess.PIPE,
+                )
+            except:
+                click.secho(
+                    f"Failed to destroy zfs dataset at {path}.", fg="red"
+                )
+                time.sleep(1)
             click.secho(f"Removed: {path}", fg="yellow")
         else:
             click.secho(f"{path} did not exist and so wasn't removed.")
@@ -394,7 +463,16 @@ def translate_poolPath_to_fullPath(zfs_path):
         mountpoint = None
         try:
             mountpoint = subprocess.check_output(
-                ["sudo", zfs, "get", "mountpoint", "-H", "-o", "value", zfs_path],
+                [
+                    "sudo",
+                    zfs,
+                    "get",
+                    "mountpoint",
+                    "-H",
+                    "-o",
+                    "value",
+                    zfs_path,
+                ],
                 encoding="utf8",
             ).strip()
         except:
