@@ -1,12 +1,9 @@
-from multiprocessing.dummy import Process
 import time
 import re
 import sys
-import uuid
 import arrow
 import threading
 import json
-import base64
 import subprocess
 import inquirer
 import traceback
@@ -16,8 +13,13 @@ import os
 import tempfile
 import click
 
+from tqdm import tqdm
+from copy import deepcopy
+from .odoo_config import get_conn_autoclose
 from gimera.repo import Repo
+from .tools import __try_to_set_owner
 from .tools import _make_sure_module_is_installed
+from .tools import __assure_gitignore
 from .tools import get_hash
 from .tools import get_directory_hash
 from .tools import sync_folder
@@ -33,6 +35,7 @@ from .tools import _update_setting
 from .tools import _get_setting
 from .tools import get_git_hash
 from .tools import start_postgres_if_local
+from .tools import _get_available_modules
 from .module_tools import _determine_affected_modules_for_ir_field_and_related
 from pathlib import Path
 from functools import partial
@@ -42,7 +45,7 @@ KEY_SHA_REVISION = "sha.revision"
 
 
 def get_all_modules_installed_by_manifest(config):
-    from .module_tools import Modules, DBModules, Module
+    from .module_tools import Modules
 
     manifest_modules = Modules().get_all_modules_installed_by_manifest()
     remove_some_modules(config, manifest_modules)
@@ -80,14 +83,13 @@ def odoo_module(config):
 @pass_config
 def abort_upgrade(config):
     click.echo("Aborting upgrade...")
-    from .module_tools import Modules, DBModules
+    from .module_tools import DBModules
 
     DBModules.abort_upgrade()
 
 
 def _get_default_modules_to_update(config):
     from .module_tools import Modules, DBModules
-    from .odoo_config import MANIFEST
 
     mods = Modules()
     module = mods.get_customs_modules("to_update")
@@ -115,7 +117,8 @@ def run_tests(ctx, config):
     started = datetime.now()
     if not config.devmode and not config.force:
         click.secho(
-            "Devmode required to run unit tests. Database will be destroyed.", fg="red"
+            "Devmode required to run unit tests. Database will be destroyed.",
+            fg="red",
         )
         sys.exit(-1)
 
@@ -137,13 +140,16 @@ def run_tests(ctx, config):
 
     def reset_db():
         if config.force:
-            Commands.invoke(ctx, "wait_for_container_postgres", missing_ok=True)
+            Commands.invoke(
+                ctx, "wait_for_container_postgres", missing_ok=True
+            )
             Commands.invoke(ctx, "reset-db")
-            Commands.invoke(ctx, "update", "", tests=False, no_dangling_check=True)
+            Commands.invoke(
+                ctx, "update", "", tests=False, no_dangling_check=True
+            )
 
     reset_db()
     from .module_tools import Module
-    from .odoo_config import customs_dir
 
     success, failed = [], []
     for module in tests:
@@ -168,7 +174,9 @@ def run_tests(ctx, config):
 
                 def run_test(file):
                     params = ["odoo", "/odoolib/unit_test.py", file]
-                    click.secho(f"Running test: {file}", fg="yellow", bold=True)
+                    click.secho(
+                        f"Running test: {file}", fg="yellow", bold=True
+                    )
                     res = __dcrun(
                         config,
                         params + ["--log-level=error", "--not-interactive"],
@@ -306,7 +314,8 @@ def _get_outdated_versioned_modules_of_deptree(modules):
                 version = tuple([int(x) for x in version.split(".")])
             except Exception:
                 click.secho(
-                    f"Broken version name in module {meta_info}: {version}", fg="red"
+                    f"Broken version name in module {meta_info}: {version}",
+                    fg="red",
                 )
                 sys.exit(-1)
             new_version = Module.get_by_name(dep).manifest_dict.get("version")
@@ -319,34 +328,23 @@ def _get_outdated_versioned_modules_of_deptree(modules):
                 assert (
                     len(odoo_version) == 2
                 ), f"Version in manifest should be like 16.0 not '{odoo_version}' as it is in {MANIFEST().path}"
-                new_version = tuple(list(map(int, odoo_version)) + list(new_version))
+                new_version = tuple(
+                    list(map(int, odoo_version)) + list(new_version)
+                )
                 del odoo_version
 
             if new_version > version:
                 # seen: if version in manifest is 14.0.1.0 and installed in
                 # 13.0 odoo then version becomes inside odoo: 13.0.14.0.10
                 # so try to match including current version:
-                odoo_version = list(map(int, str(MANIFEST()["version"]).split(".")))
+                odoo_version = list(
+                    map(int, str(MANIFEST()["version"]).split("."))
+                )
                 new_version = tuple(odoo_version + list(new_version))
                 if new_version != version:
                     yield dep
 
 
-def _get_available_modules(ctx, param, incomplete):
-    from .odoo_config import MANIFEST
-
-    try:
-        manifest = MANIFEST()
-        if not manifest:
-            raise Exception("no manifest")
-    except:
-        return []
-    modules = manifest["install"]
-    if incomplete:
-        modules = [x for x in modules if incomplete in x]
-    return sorted(modules)
-
-
 @odoo_module.command(name="UPDATE")
 @click.option(
     "--no-dangling-check",
@@ -360,10 +358,14 @@ def _get_available_modules(ctx, param, incomplete):
     is_flag=True,
     help="Can happen if per update fields are removed and views still referencing this field.",
 )
-@click.option("--i18n", default=False, is_flag=True, help="Overwrite Translations")
+@click.option(
+    "--i18n", default=False, is_flag=True, help="Overwrite Translations"
+)
 @pass_config
 @click.pass_context
-def update2(ctx, config, no_dangling_check, non_interactive, recover_view_error, i18n):
+def update2(
+    ctx, config, no_dangling_check, non_interactive, recover_view_error, i18n
+):
     conn = config.get_odoo_conn()
     revision = _get_setting(conn, KEY_SHA_REVISION)
     Commands.invoke(
@@ -378,6 +380,25 @@ def update2(ctx, config, no_dangling_check, non_interactive, recover_view_error,
     )
 
 
+@odoo_module.command()
+@click.argument(
+    "module", nargs=-1, required=False, shell_complete=_get_available_modules
+)
+@pass_config
+@click.pass_context
+def make_sure_module_is_installed(ctx, config, module):
+    from .module_tools import DBModules
+
+    if not module:
+        abort("Please provide some modules.")
+    installed_modules = DBModules.get_all_installed_modules()
+    for mod in module:
+        if mod not in installed_modules:
+            ctx.invoke(update, module=[mod])
+        else:
+            click.secho(f"Module '{mod}' already installed.", fg="green")
+
+
 @odoo_module.command()
 @click.argument(
     "module", nargs=-1, required=False, shell_complete=_get_available_modules
@@ -423,7 +444,9 @@ def update2(ctx, config, no_dangling_check, non_interactive, recover_view_error,
     help="Not checking for dangling modules",
 )
 @click.option("--tests", default=False, is_flag=True, help="Runs tests")
-@click.option("--i18n", default=False, is_flag=True, help="Overwrite Translations")
+@click.option(
+    "--i18n", default=False, is_flag=True, help="Overwrite Translations"
+)
 @click.option("--no-install-server-wide-first", default=False, is_flag=True)
 @click.option("--no-extra-addons-paths", is_flag=True)
 @click.option(
@@ -434,7 +457,9 @@ def update2(ctx, config, no_dangling_check, non_interactive, recover_view_error,
 )
 @click.option("--server-wide-modules")
 @click.option(
-    "--uninstall", is_flag=True, help="Executes just uninstallation of modules."
+    "--uninstall",
+    is_flag=True,
+    help="Executes just uninstallation of modules.",
 )
 @click.option(
     "--test-tags",
@@ -476,6 +501,16 @@ def update2(ctx, config, no_dangling_check, non_interactive, recover_view_error,
     is_flag=False,
     help="directs stdout to given file",
 )
+@click.option(
+    "--no-scripts",
+    is_flag=True,
+)
+@click.option(
+    "-P",
+    "--no-progress",
+    is_flag=True,
+    help="No progress, but allows interactive debugging",
+)
 @pass_config
 @click.pass_context
 def update(
@@ -503,6 +538,8 @@ def update(
     recover_view_error=False,
     no_outdated_modules=False,
     stdout=False,
+    no_scripts=False,
+    no_progress=False,
 ):
     """
     Just custom modules are updated, never the base modules (e.g. prohibits adding old stock-locations)
@@ -527,6 +564,9 @@ def update(
     """
     started = arrow.get()
 
+    if float(config.odoo_version) < 14.0:
+        no_progress = True
+
     param_module = module
 
     if recover_view_error and not non_interactive:
@@ -554,13 +594,26 @@ def update(
     """,
         fg="green",
     )
-    from .module_tools import Modules, DBModules, Module
+    from .module_tools import DBModules, Module
     from .odoo_config import MANIFEST
+    from .odoo_config import customs_dir
 
     if test_tags and default_test_tags:
         abort("Conflict: parameter test-tags and default-test-tags")
 
+    update_log_file = customs_dir() / "update.log"
+    update_log_file.write_text("")
+    __try_to_set_owner(
+        int(config.owner_uid),
+        update_log_file,
+        verbose=True,
+    )
     start_postgres_if_local(ctx, config)
+    manifest = MANIFEST()
+    if manifest.get("before-odoo-update", []) and not no_scripts:
+        if os.getenv("NO_BEFORE_ODOO_COMMAND") != "1":
+            click.secho("Running before-odoo-update", fg="yellow")
+            _exec_commands(ctx, config, manifest.get("before-odoo-update", []))
 
     def _perform_install(module):
         if since_git_sha and module:
@@ -587,11 +640,15 @@ def update(
 
         if not no_restart:
             if config.use_docker:
-                Commands.invoke(ctx, "kill", machines=get_services(config, "odoo_base"))
+                Commands.invoke(
+                    ctx, "kill", machines=get_services(config, "odoo_base")
+                )
                 if config.run_redis:
                     Commands.invoke(ctx, "up", machines=["redis"], daemon=True)
                 if config.run_postgres:
-                    Commands.invoke(ctx, "up", machines=["postgres"], daemon=True)
+                    Commands.invoke(
+                        ctx, "up", machines=["postgres"], daemon=True
+                    )
                 Commands.invoke(ctx, "wait_for_container_postgres")
 
         if not no_dangling_check:
@@ -607,7 +664,8 @@ def update(
         def _effective_test_tags():
             if default_test_tags:
                 return ",".join(
-                    f"at_install/{x},post_install{x},standard/{x}" for x in module
+                    f"at_install/{x},post_install{x},standard/{x}"
+                    for x in module
                 )
             else:
                 return test_tags or ""
@@ -619,9 +677,35 @@ def update(
             ).write_text(datetime.now().strftime(DTF))
 
         def _technically_update(modules):
+            if not no_progress:
+                progress = ProgressBarDaemon(interval=0.2)
+                progress.start()
+            output = []
+
+            def _eval_progress(line):
+                def extract_progress(text):
+                    """Extracts module name and progress values from text like 'Loading module documents_project (192/510)'."""
+                    match = re.search(
+                        r"Loading module ([\w_]+) \((\d+)/(\d+)\)", text
+                    )
+                    if match:
+                        module_name, current, total = match.groups()
+                        return module_name, int(current), int(total)
+                    return None, None, None
+
+                module, prog, total = extract_progress(line)
+                if total:
+                    prog = int(prog)
+                    total = int(total)
+                    progress.update_progress(module, prog, total)
+                output.append(line)
+
             try:
                 modules = list(
-                    map(lambda x: x.name if isinstance(x, Module) else x, modules)
+                    map(
+                        lambda x: x.name if isinstance(x, Module) else x,
+                        modules,
+                    )
                 )
                 params = [",".join(modules)]
                 if no_extra_addons_paths:
@@ -649,7 +733,11 @@ def update(
                     _exec_update(
                         config,
                         params,
-                        non_interactive=non_interactive if not stdout else True,
+                        non_interactive=not no_progress,  # because of progressbar; non_interactive if not stdout else True,
+                        write_to_console=_eval_progress
+                        if not no_progress
+                        else None,
+                        # write_to_console=lambda line: progress.write(line),
                     )
                 )
                 if len(rc) == 1:
@@ -660,7 +748,12 @@ def update(
                 if stdout:
                     Path(stdout).write_text(output)
 
+                if not no_progress:
+                    update_log_file.write_text("".join(output))
+
                 if rc:
+                    if not no_progress:
+                        click.secho(update_log_file.read_text())
                     if recover_view_error:
                         try:
                             _try_to_recover_view_error(config, output)
@@ -669,7 +762,9 @@ def update(
                         except Exception as ex:
                             raise UpdateException(module) from ex
                         else:
-                            raise Exception(f"Error at update - please check logs")
+                            raise Exception(
+                                f"Error at update - please check logs"
+                            )
                     else:
                         raise UpdateException(module)
 
@@ -679,10 +774,18 @@ def update(
                 raise
             except Exception as ex:
                 click.echo(traceback.format_exc())
-                ctx.invoke(show_install_state, suppress_error=no_dangling_check)
+                ctx.invoke(
+                    show_install_state, suppress_error=no_dangling_check
+                )
                 raise Exception(
-                    ("Error at /update_modules.py - " "aborting update process.")
+                    (
+                        "Error at /update_modules.py - "
+                        "aborting update process."
+                    )
                 ) from ex
+            finally:
+                if not no_progress:
+                    progress.stop()
 
         trycount = 0
         max_try_count = 5
@@ -722,7 +825,6 @@ def update(
     if not uninstall:
         _perform_install(module)
 
-    manifest = MANIFEST()
     _uninstall_devmode_modules(ctx, config, manifest)
 
     all_modules = (
@@ -732,26 +834,50 @@ def update(
     )
 
     if uninstall or all_modules:
-        _uninstall_marked_modules(ctx, config, manifest.get("uninstall", []))
+        _uninstall_marked_modules(
+            ctx, config, manifest.get("uninstall", []), no_restart
+        )
 
     # check danglings
     if not no_dangling_check and all_modules:
         ctx.invoke(show_install_state, suppress_error=False)
 
     if check_install_state:
-        _do_check_install_state(ctx, config, module, all_modules, no_dangling_check)
+        _do_check_install_state(
+            ctx, config, module, all_modules, no_dangling_check
+        )
 
     _set_sha(config)
 
     duration = (arrow.get() - started).total_seconds()
     date = arrow.get().strftime("%Y-%m-%d %H:%M:%S")
     click.secho(f"Update done at {date} - duration {duration}s", fg="yellow")
+    if not no_progress:
+        click.secho("Update-log here: ./update.log")
+        __assure_gitignore(
+            customs_dir() / ".gitignore",
+            update_log_file.relative_to(customs_dir()),
+        )
+
+
+def _exec_commands(ctx, config, commands):
+    for command in commands:
+        cmd = (
+            [sys.executable, sys.argv[0]]
+            + ["-p", config.PROJECT_NAME]
+            + command
+        )
+        env = deepcopy(os.environ)
+        env["NO_BEFORE_ODOO_COMMAND"] = "1"
+        subprocess.run(cmd, check=True, env=env)
 
 
 def _uninstall_devmode_modules(ctx, config, manifest):
     if config.devmode and manifest.get("devmode_uninstall", []):
         for mod in manifest["devmode_uninstall"]:
-            _uninstall_marked_modules(ctx, config, manifest["devmode_uninstall"])
+            _uninstall_marked_modules(
+                ctx, config, manifest["devmode_uninstall"], False
+            )
 
 
 def _set_sha(config):
@@ -788,17 +914,21 @@ def _try_to_recover_view_error(config, output):
     lines = output.splitlines()
 
     for i, line in enumerate(lines):
-        match = re.findall('Field "([^"]*?)" does not exist in model "([^"]*?)"', line)
+        match = re.findall(
+            'Field "([^"]*?)" does not exist in model "([^"]*?)"', line
+        )
         if match:
             field, model = match[0]
-            affected_modules = _determine_affected_modules_for_ir_field_and_related(
-                config, field, model
+            affected_modules = (
+                _determine_affected_modules_for_ir_field_and_related(
+                    config, field, model
+                )
             )
             raise RepeatUpdate(affected_modules)
 
 
 def show_dangling():
-    from .module_tools import Modules, DBModules, Module
+    from .module_tools import DBModules
 
     dangling = list(DBModules.get_dangling_modules())
     if dangling:
@@ -808,8 +938,10 @@ def show_dangling():
     return bool(dangling)
 
 
-def _do_check_install_state(ctx, config, module, all_modules, no_dangling_check):
-    from .module_tools import Modules, DBModules, Module
+def _do_check_install_state(
+    ctx, config, module, all_modules, no_dangling_check
+):
+    from .module_tools import DBModules
 
     if all_modules:
         ctx.invoke(
@@ -832,11 +964,13 @@ def _do_check_install_state(ctx, config, module, all_modules, no_dangling_check)
 
 
 def _do_dangling_check(ctx, config, dangling_modules, non_interactive):
-    from .module_tools import Modules, DBModules, Module
+    from .module_tools import DBModules
 
     if any(x[1] == "uninstallable" for x in DBModules.get_dangling_modules()):
         if non_interactive:
-            abort("Danling modules exist. Provide --no-dangling-check otherwise.")
+            abort(
+                "Danling modules exist. Provide --no-dangling-check otherwise."
+            )
         for x in DBModules.get_dangling_modules():
             click.echo("{}: {}".format(*x[:2]))
         if (
@@ -880,11 +1014,11 @@ def uninstall(ctx, config, modules):
     _uninstall_marked_modules(ctx, config, modules)
 
 
-def _uninstall_marked_modules(ctx, config, modules):
+def _uninstall_marked_modules(ctx, config, modules, no_restart=False):
     """
     Checks for file "uninstall" in customs root and sets modules to uninstalled.
     """
-    from .module_tools import Modules, DBModules, Module
+    from .module_tools import DBModules, Module
     from .module_tools import NotInAddonsPath
 
     assert not isinstance(modules, str)
@@ -904,38 +1038,68 @@ def _uninstall_marked_modules(ctx, config, modules):
     manifest_modules = get_all_modules_installed_by_manifest(config)
 
     uninstalled = False
+    effective_uninstall = []
+    errors = []
     for module in modules:
         try:
             objmod = Module.get_by_name(module)
             for desc in objmod.descendants:
                 if desc in manifest_modules:
-                    if not Module.get_by_name(desc).manifest_dict.get("auto_install"):
-                        abort(
-                            f"{objmod.name} has {desc.name} as descendant which is still in the install section"
+                    if not Module.get_by_name(desc).manifest_dict.get(
+                        "auto_install"
+                    ):
+                        errors.append(
+                            (
+                                f"{objmod.name} has {desc.name} as descendant "
+                                "which is still in the install section"
+                            )
                         )
         except NotInAddonsPath:
             pass
-
-        click.secho(f"Uninstall {module}", fg="red")
-        lib_shell(
-            config,
-            (
-                "self.env['ir.module.module'].search(["
-                f"('name', '=', '{module}'),"
-                "('state', 'in', "
-                "['to upgrade', 'to install', 'installed']"
-                ")]).module_uninstall()\n"
-                "self.env.cr.commit()"
-            ),
+        else:
+            effective_uninstall.append(module)
+    if errors:
+        abort("\n".join(errors))
+    if effective_uninstall:
+        module_comma = ",".join(
+            map(lambda x: f"'{x}'", sorted(effective_uninstall))
         )
+        if len(effective_uninstall) > 10:
+            click.secho(
+                f"Uninstalling {len(effective_uninstall)} modules", fg="red"
+            )
+        else:
+            click.secho(f"Uninstall {module_comma}", fg="red")
+        module_comma = f"[{module_comma}]"
+        cmd = f"""
+modules = self.env['ir.module.module'].search([('state', 'in', ['to upgrade', 'to install', 'installed'])])
+modules = modules.filtered(lambda x: x.name in {module_comma})
+for module in modules:
+    try:
+        print("-----------------------------------------------------------")
+        print("Uninstalling: " + module.name)
+        print("-----------------------------------------------------------")
+        module.module_uninstall()
+        self.env.cr.commit()
+    except Exception as ex:
+        print("-----------------------------------------------------------")
+        print("ERROR UNINSTALL " + module.name)
+        print(ex)
+        print("-----------------------------------------------------------")
+    finally:
+        env.clear()
+        env.cr.rollback()
+            """
+        lib_shell(config, cmd)
         del module
         uninstalled = True
 
-    if uninstalled:
+    if uninstalled and not no_restart:
         Commands.invoke(ctx, "restart", machines=["odoo"])
 
     modules = [x for x in modules if DBModules.is_module_installed(x)]
     if modules:
+        click.secho(cmd, fg="yellow")
         abort(f"Failed to uninstall: {','.join(modules)}")
 
 
@@ -972,7 +1136,9 @@ def update_i18n(ctx, config, module, no_restart):
     except Exception:
         click.echo(traceback.format_exc())
         ctx.invoke(show_install_state, suppress_error=True)
-        raise Exception("Error at /update_modules.py - aborting update process.")
+        raise Exception(
+            "Error at /update_modules.py - aborting update process."
+        )
 
     if not no_restart:
         Commands.invoke(ctx, "restart", machines=["odoo"])
@@ -1030,8 +1196,10 @@ def show_conflicting_modules(config):
     )
 
 
-def _exec_update(config, params, non_interactive=False, stdout=False):
-    params = ["odoo_update", "/update_modules.py"] + params
+def _exec_update(
+    config, params, non_interactive=False, stdout=False, write_to_console=True
+):
+    params = ["odoo_update", "/odoolib/update_modules.py"] + params
     if not non_interactive:
         yield __cmd_interactive(
             config,
@@ -1045,7 +1213,13 @@ def _exec_update(config, params, non_interactive=False, stdout=False):
         )
     else:
         try:
-            returncode, output = __dcrun(config, list(params), returnproc=True)
+            returncode, output = __dcrun(
+                config,
+                list(params),
+                returnproc=True,
+                interactive=False,
+                write_to_console=write_to_console,
+            )
             yield returncode
             yield output
         except subprocess.CalledProcessError as ex:
@@ -1054,7 +1228,6 @@ def _exec_update(config, params, non_interactive=False, stdout=False):
 
 def _get_unittests_from_module(module_name):
     from .module_tools import Module
-    from .odoo_config import MANIFEST_FILE
 
     testfiles = []
     module = Module.get_by_name(module_name)
@@ -1076,9 +1249,24 @@ def _get_all_unittest_files(config):
 
 
 @odoo_module.command()
+@click.option(
+    "-m",
+    "--manifest",
+    help="Evaluate 'tests' section in MANIFEST",
+    is_flag=True,
+)
 @pass_config
-def list_unit_test_files(config):
+def list_unit_test_files(config, manifest):
+    from .odoo_config import MANIFEST
+
     files = _get_all_unittest_files(config)
+
+    if manifest:
+        tests = MANIFEST().get("tests", [])
+        files = list(
+            filter(lambda file: str(Path(file).parent.parent) in tests, files)
+        )
+
     click.secho("!!!")
     for file in files:
         click.secho(file)
@@ -1107,14 +1295,20 @@ def list_robot_test_files(config):
 @click.option("--log", is_flag=True)
 @pass_config
 def unittest(
-    config, file, remote_debug, wait_for_remote, non_interactive, output_json, tags, log
+    config,
+    file,
+    remote_debug,
+    wait_for_remote,
+    non_interactive,
+    output_json,
+    tags,
+    log,
 ):
     """
     Collects unittest files and offers to run
     """
-    from .odoo_config import MANIFEST, MANIFEST_FILE, customs_dir
+    from .odoo_config import customs_dir
     from .module_tools import Module
-    from pathlib import Path
 
     if file and "/" not in file:
         try:
@@ -1194,11 +1388,13 @@ def unittest(
 
         if passed:
             click.secho(
-                tabulate(passed, headers="keys", tablefmt="fancy_grid"), fg="green"
+                tabulate(passed, headers="keys", tablefmt="fancy_grid"),
+                fg="green",
             )
         if errors:
             click.secho(
-                tabulate(errors, headers="keys", tablefmt="fancy_grid"), fg="red"
+                tabulate(errors, headers="keys", tablefmt="fancy_grid"),
+                fg="red",
             )
     if errors:
         sys.exit(-1)
@@ -1213,7 +1409,6 @@ def generate_update_command(ctx, config):
 
 
 def _get_changed_files(git_sha):
-    from .module_tools import Module
     from .tools import git_diff_files
 
     cwd = os.getcwd()
@@ -1316,10 +1511,7 @@ def _get_directory_hash(path):
 @pass_config
 @click.pass_context
 def list_descendants(ctx, config, module, customs, following):
-    from .module_tools import Modules, DBModules, Module
-    from .module_tools import NotInAddonsPath
-    from .odoo_config import customs_dir
-    from .consts import FILE_DIRHASHES
+    from .module_tools import Module
 
     mod = Module.get_by_name(module)
     res = mod.descendants
@@ -1337,10 +1529,8 @@ def list_deps(ctx, config, module, no_cache, customs):
     import arrow
 
     started = arrow.get()
-    from .module_tools import Modules, DBModules, Module
+    from .module_tools import Modules, Module
     from .module_tools import NotInAddonsPath
-    from .odoo_config import customs_dir
-    from .consts import FILE_DIRHASHES
 
     click.secho("Loading Modules...", fg="yellow")
     modules = Modules()
@@ -1355,7 +1545,12 @@ def list_deps(ctx, config, module, no_cache, customs):
     for module in module:
         data = {"modules": []}
         data["modules"] = sorted(
-            list(map(lambda x: x.name, modules.get_module_flat_dependency_tree(module)))
+            list(
+                map(
+                    lambda x: x.name,
+                    modules.get_module_flat_dependency_tree(module),
+                )
+            )
             + [module.name]
         )
 
@@ -1426,7 +1621,7 @@ def list_deps(ctx, config, module, no_cache, customs):
 
 def _filter_customs(modules):
     modules2 = {}
-    from .module_tools import DBModules, Module
+    from .module_tools import Module
 
     def _filter(module_name):
         mod = Module.get_by_name(module_name)
@@ -1525,7 +1720,6 @@ def _get_changed_modules(git_sha):
 def list_installed_modules(config, fix_not_in_manifest, only_customs):
     from .module_tools import DBModules, Module
     from .module_tools import NotInAddonsPath
-    from .odoo_config import customs_dir
     from .odoo_config import MANIFEST
 
     collected = []
@@ -1537,13 +1731,17 @@ def list_installed_modules(config, fix_not_in_manifest, only_customs):
         try:
             mod = Module.get_by_name(module)
         except (Module.IsNot, NotInAddonsPath):
-            click.secho(f"Ignoring {module} - not found in source", fg="yellow")
+            click.secho(
+                f"Ignoring {module} - not found in source", fg="yellow"
+            )
             continue
         if only_customs:
             try:
                 parts = mod.path.parts
             except Module.IsNot:
-                click.secho(f"Ignoring {module} - not found in source", fg="yellow")
+                click.secho(
+                    f"Ignoring {module} - not found in source", fg="yellow"
+                )
                 continue
             if not mod.is_customs:
                 continue
@@ -1573,8 +1771,7 @@ def list_installed_modules(config, fix_not_in_manifest, only_customs):
 @click.pass_context
 @pass_config
 def zip(config, ctx, module):
-    from .module_tools import Modules, Module
-    from .odoo_config import customs_dir
+    from .module_tools import Module
 
     try:
         mod = Module.get_by_name(module)
@@ -1588,7 +1785,9 @@ def zip(config, ctx, module):
     zipfile = Path(os.getcwd()) / f"{module}.zip"
     if zipfile.exists():
         zipfile.unlink()
-    subprocess.check_call(["/usr/bin/zip", "-r", zipfile.name, "."], cwd=module_path)
+    subprocess.check_call(
+        ["/usr/bin/zip", "-r", zipfile.name, "."], cwd=module_path
+    )
     shutil.move(Path(module_path / zipfile.name), zipfile)
     click.secho(f"Created zipfile: {zipfile}", fg="green")
 
@@ -1618,6 +1817,53 @@ def scan_addons_paths(config, ctx):
         click.secho(str(item))
 
 
+class ProgressBarDaemon(threading.Thread):
+    def __init__(self, interval=1.0):
+        super().__init__(daemon=True)
+        self.interval = interval
+        self.running = True
+        self.total = None
+        self.pbar = tqdm(
+            total=100,
+            bar_format="\033[95m{l_bar}{bar}\033[0m {n_fmt}/{total_fmt} [{elapsed}<{remaining}]",
+            dynamic_ncols=True,
+        )
+        self.pbar.set_description("Waiting for update process")
+
+    def run(self):
+        while self.running:
+            self.pbar.refresh()
+            time.sleep(self.interval)
+
+    def get_total(self):
+        return self.get_to_install_modules()
+
+    def get_to_install_modules(self):
+        with get_conn_autoclose() as cr:
+            cr.execute(
+                "select count(*) from ir_module_module where state in ('to install', 'to upgrade')"
+            )
+            record = cr.fetchone()
+        return record[0]
+
+    def update_progress(self, module, progress, total):
+        self.pbar.total = total
+        self.pbar.n = progress
+        text = module.ljust(40)
+        self.pbar.set_description(text)
+
+    def write(self, line):
+        if self.pbar:
+            self.pbar.write(line)
+        else:
+            print(line)
+
+    def stop(self):
+        self.running = False
+        if self.pbar:
+            self.pbar.close()
+
+
 Commands.register(update)
 Commands.register(show_install_state)
 Commands.register(uninstall)
